# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.forms import widgets
from django.forms.fields import CharField
from django.forms.models import ModelForm
from django.utils.translation import ungettext_lazy, ugettext_lazy as _
from django.utils.text import Truncator
from django.utils.html import format_html
from django.forms.fields import IntegerField
from django.template.loader import select_template

from cms.plugin_pool import plugin_pool
from cmsplugin_cascade.forms import ManageChildrenFormMixin
from cmsplugin_cascade.fields import GlossaryField
from cmsplugin_cascade.link.forms import TextLinkFormMixin
from cmsplugin_cascade.widgets import NumberInputWidget
from cmsplugin_cascade.bootstrap3.buttons import BootstrapButtonMixin
from cmsplugin_cascade.plugin_base import TransparentWrapper, TransparentContainer

from shop.conf import app_settings
from shop.cascade.plugin_base import ShopPluginBase


class ProcessBarForm(ManageChildrenFormMixin, ModelForm):
    num_children = IntegerField(min_value=1, initial=1,
        widget=NumberInputWidget(attrs={'size': '3', 'style': 'width: 5em;'}),
        label=_("Steps"),
        help_text=_("Number of steps for this proceed bar."))


class ProcessBarPlugin(TransparentWrapper, ShopPluginBase):
    name = _("Process Bar")
    form = ProcessBarForm
    parent_classes = ('BootstrapColumnPlugin',)
    direct_child_classes = ('ProcessStepPlugin',)
    require_parent = True
    allow_children = True

    @classmethod
    def get_identifier(cls, instance):
        identifier = super(ProcessBarPlugin, cls).get_identifier(instance)
        num_cols = instance.get_children().count()
        content = ungettext_lazy('with {} page', 'with {} pages', num_cols).format(num_cols)
        return format_html('{0}{1}', identifier, content)

    def get_render_template(self, context, instance, placeholder):
        template_names = [
            '{}/checkout/process-bar.html'.format(app_settings.APP_LABEL),
            'shop/checkout/process-bar.html',
        ]
        return select_template(template_names)

    def render(self, context, instance, placeholder):
        self.super(ProcessBarPlugin, self).render(context, instance, placeholder)
        num_children = instance.get_num_children()
        if num_children > 0:
            context['step_css_width'] = '{:3.2f}%'.format(100. / num_children)
        return context

    def save_model(self, request, obj, form, change):
        wanted_children = int(form.cleaned_data.get('num_children'))
        super(ProcessBarPlugin, self).save_model(request, obj, form, change)
        self.extend_children(obj, wanted_children, ProcessStepPlugin)

plugin_pool.register_plugin(ProcessBarPlugin)


class ProcessStepPlugin(TransparentContainer, ShopPluginBase):
    name = _("Process Step")
    direct_parent_classes = parent_classes = ('ProcessBarPlugin',)
    require_parent = True
    allow_children = True
    alien_child_classes = True
    render_template = 'cascade/generic/wrapper.html'
    step_title = GlossaryField(
        widgets.TextInput(attrs={'size': 50}),
        label=_("Step Title")
    )

    @classmethod
    def get_identifier(cls, obj):
        identifier = super(ProcessStepPlugin, cls).get_identifier(obj)
        content = obj.glossary.get('step_title', '')
        if content:
            content = Truncator(content).words(3, truncate=' ...')
        else:
            content = obj.get_position_in_placeholder()
        return format_html('{0}{1}', identifier, content)

plugin_pool.register_plugin(ProcessStepPlugin)


class ProcessNextStepForm(TextLinkFormMixin, ModelForm):
    link_content = CharField(required=False, label=_("Button Content"), widget=widgets.TextInput())

    def __init__(self, raw_data=None, *args, **kwargs):
        instance = kwargs.get('instance')
        initial = instance and dict(instance.glossary) or {}
        initial.update(kwargs.pop('initial', {}))
        kwargs.update(initial=initial)
        super(ProcessNextStepForm, self).__init__(raw_data, *args, **kwargs)


class ProcessNextStepPlugin(BootstrapButtonMixin, ShopPluginBase):
    name = _("Next Step Button")
    parent_classes = ('ProcessStepPlugin',)
    form = ProcessNextStepForm
    fields = ('link_content', 'glossary')
    ring_plugin = 'ProcessNextStepPlugin'
    glossary_field_order = ('button_type', 'button_size', 'button_options', 'quick_float',
                            'icon_align', 'icon_font', 'symbol')

    class Media:
        css = {'all': ('cascade/css/admin/bootstrap.min.css',
                       'cascade/css/admin/bootstrap-theme.min.css',
                       'cascade/css/admin/iconplugin.css',)}
        js = ['shop/js/admin/nextstepplugin.js']

    @classmethod
    def get_identifier(cls, obj):
        identifier = super(ProcessNextStepPlugin, cls).get_identifier(obj)
        content = obj.glossary.get('link_content', '')
        return format_html('{0}{1}', identifier, content)

    def get_render_template(self, context, instance, placeholder):
        template_names = [
            '{}/checkout/process-next-step.html'.format(app_settings.APP_LABEL),
            'shop/checkout/process-next-step.html',
        ]
        return select_template(template_names)

plugin_pool.register_plugin(ProcessNextStepPlugin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import enum
import six

from django.conf import settings
from django.db import models
from django.utils.six import python_2_unicode_compatible, string_types
from django.utils.translation import ugettext_lazy as _, ugettext


postgresql_engine_names = [
    'django.db.backends.postgresql',
    'django.db.backends.postgresql_psycopg2',
]

if settings.DATABASES['default']['ENGINE'] in postgresql_engine_names:
    from django.contrib.postgres.fields import JSONField as _JSONField
else:
    from jsonfield.fields import JSONField as _JSONField


class JSONField(_JSONField):
    def __init__(self, *args, **kwargs):
        kwargs.update({'default': {}})
        super(JSONField, self).__init__(*args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super(JSONField, self).deconstruct()
        del kwargs['default']
        return name, path, args, kwargs


class ChoiceEnumMeta(enum.EnumMeta):
    def __call__(cls, value, *args, **kwargs):
        if isinstance(value, string_types):
            try:
                value = cls.__members__[value]
            except KeyError:
                pass  # let the super method complain
        return super(ChoiceEnumMeta, cls).__call__(value, *args, **kwargs)


@python_2_unicode_compatible
class ChoiceEnum(six.with_metaclass(ChoiceEnumMeta, enum.Enum)):
    """
    Utility class to handle choices in Django model fields
    """
    def __str__(self):
        return ugettext('.'.join((self.__class__.__name__, self.name)))

    @classmethod
    def default(cls):
        try:
            return next(iter(cls))
        except StopIteration:
            return None

    @classmethod
    def choices(cls):
        choices = [(c.value, str(c)) for c in cls]
        return choices


class ChoiceEnumField(models.PositiveSmallIntegerField):
    description = _("Customer recognition state")

    def __init__(self, *args, **kwargs):
        self.enum_type = kwargs.pop('enum_type', ChoiceEnum)  # fallback is required form migrations
        if not issubclass(self.enum_type, ChoiceEnum):
            raise ValueError("enum_type must be a subclass of `ChoiceEnum`.")
        kwargs.update(choices=self.enum_type.choices())
        kwargs.setdefault('default', self.enum_type.default())
        super(ChoiceEnumField, self).__init__(*args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super(ChoiceEnumField, self).deconstruct()
        if 'choices' in kwargs:
            del kwargs['choices']
        if kwargs['default'] is self.enum_type.default():
            del kwargs['default']
        elif isinstance(kwargs['default'], self.enum_type):
            kwargs['default'] = kwargs['default'].value
        return name, path, args, kwargs

    def from_db_value(self, value, expression, connection, context):
        try:
            return self.enum_type(value)
        except ValueError:
            return value

    def get_prep_value(self, state):
        if isinstance(state, self.enum_type):
            return state.value
        if isinstance(state, int):
            return state
        raise ValueError("Value must be of type {}".format(self.enum_type))

    def to_python(self, state):
        return self.enum_type(state)

    def value_to_string(self, obj):
        if not isinstance(obj, self.enum_type):
            raise ValueError("Value must be of type {}".format(self.enum_type))
        return obj.name



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib.auth import get_user_model, authenticate, login
from django.contrib.sites.shortcuts import get_current_site
from django.core.exceptions import ValidationError
from django.forms import widgets, ModelForm
from django.template import Context
from django.template.loader import select_template
from django.utils.translation import ugettext_lazy as _

from djng.forms import fields, NgModelFormMixin, NgFormValidationMixin
from djng.styling.bootstrap3.forms import Bootstrap3ModelForm

from shop.conf import app_settings
from shop.models.customer import CustomerModel
from .base import UniqueEmailValidationMixin


class RegisterUserForm(NgModelFormMixin, NgFormValidationMixin, UniqueEmailValidationMixin, Bootstrap3ModelForm):
    form_name = 'register_user_form'
    scope_prefix = 'form_data'
    field_css_classes = 'input-group has-feedback'

    email = fields.EmailField(label=_("Your e-mail address"))

    preset_password = fields.BooleanField(
        label=_("Preset password"),
        widget=widgets.CheckboxInput(),
        required=False,
        help_text=_("Send a randomly generated password to your e-mail address."))

    password1 = fields.CharField(
        label=_("Choose a password"),
        widget=widgets.PasswordInput,
        min_length=6,
        help_text=_("Minimum length is 6 characters."),
    )

    password2 = fields.CharField(
        label=_("Repeat password"),
        widget=widgets.PasswordInput,
        min_length=6,
        help_text=_("Confirm password."),
    )

    class Meta:
        model = CustomerModel
        fields = ['email', 'password1', 'password2']

    def __init__(self, data=None, instance=None, *args, **kwargs):
        if data and data.get('preset_password', False):
            pwd_length = max(self.base_fields['password1'].min_length, 8)
            password = get_user_model().objects.make_random_password(pwd_length)
            data['password1'] = data['password2'] = password
        super(RegisterUserForm, self).__init__(data=data, instance=instance, *args, **kwargs)

    def clean(self):
        cleaned_data = super(RegisterUserForm, self).clean()
        # check for matching passwords
        if 'password1' not in self.errors and 'password2' not in self.errors:
            if cleaned_data['password1'] != cleaned_data['password2']:
                msg = _("Passwords do not match")
                raise ValidationError(msg)
        return cleaned_data

    def save(self, request=None, commit=True):
        self.instance.user.is_active = True
        self.instance.user.email = self.cleaned_data['email']
        self.instance.user.set_password(self.cleaned_data['password1'])
        self.instance.recognize_as_registered(request, commit=False)
        customer = super(RegisterUserForm, self).save(commit)
        password = self.cleaned_data['password1']
        if self.cleaned_data['preset_password']:
            self._send_password(request, customer.user, password)
        user = authenticate(username=customer.user.username, password=password)
        login(request, user)
        return customer

    def _send_password(self, request, user, password):
        current_site = get_current_site(request)
        context = Context({
            'site_name': current_site.name,
            'absolute_base_uri': request.build_absolute_uri('/'),
            'email': user.email,
            'password': password,
            'user': user,
        })
        subject = select_template([
            '{}/email/register-user-subject.txt'.format(app_settings.APP_LABEL),
            'shop/email/register-user-subject.txt',
        ]).render(context)
        # Email subject *must not* contain newlines
        subject = ''.join(subject.splitlines())
        body = select_template([
            '{}/email/register-user-body.txt'.format(app_settings.APP_LABEL),
            'shop/email/register-user-body.txt',
        ]).render(context)
        user.email_user(subject, body)


class ContinueAsGuestForm(ModelForm):
    """
    Handles Customer's decision to order as guest.
    """
    form_name = 'continue_as_guest_form'
    scope_prefix = 'form_data'

    class Meta:
        model = CustomerModel
        fields = ()  # this form doesn't show any fields

    def save(self, request=None, commit=True):
        self.instance.recognize_as_guest(request, commit=False)
        self.instance.user.is_active = app_settings.GUEST_IS_ACTIVE_USER
        if self.instance.user.is_active:
            # set a usable password, otherwise the user later can not reset its password
            password = get_user_model().objects.make_random_password(length=30)
            self.instance.user.set_password(password)
        return super(ContinueAsGuestForm, self).save(commit)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.conf.urls import url
from rest_auth.views import PasswordChangeView
from shop.forms.auth import RegisterUserForm, ContinueAsGuestForm
from shop.views.auth import AuthFormsView, LoginView, LogoutView, PasswordResetView

urlpatterns = [
    url(r'^password/reset/?$', PasswordResetView.as_view(),
        name='password-reset'),
    url(r'^login/?$', LoginView.as_view(),
        name='login'),
    url(r'^register/?$', AuthFormsView.as_view(form_class=RegisterUserForm),
        name='register-user'),
    url(r'^continue/?$', AuthFormsView.as_view(form_class=ContinueAsGuestForm),
        name='continue-as-guest'),

    url(r'^logout/?$', LogoutView.as_view(),
        name='logout'),
    url(r'^password/change/?$', PasswordChangeView.as_view(),
        name='password-change'),
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.core.exceptions import ImproperlyConfigured
from django.forms import widgets
from django.template import engines, TemplateDoesNotExist
from django.template.loader import select_template
from django.utils.html import mark_safe
from django.utils.translation import ugettext_lazy as _

from cms.apphook_pool import apphook_pool
from cms.plugin_pool import plugin_pool
from cmsplugin_cascade.fields import GlossaryField

from shop.conf import app_settings
from .plugin_base import ShopPluginBase


class BreadcrumbPlugin(ShopPluginBase):
    name = _("Breadcrumb")
    CHOICES = (('default', _("Default Breadcrumb")), ('soft-root', _("“Soft-Root” Breadcrumb")),
               ('catalog', _("With Catalog Count")), ('empty', _("Hidden Breadcrumb")),)
    parent_classes = ()
    allow_children = None

    render_type = GlossaryField(
        widgets.RadioSelect(choices=CHOICES),
        label=_("Render as"),
        initial='default',
        help_text=_("Render an alternative Breadcrumb"),
    )

    @classmethod
    def get_identifier(cls, instance):
        render_type = instance.glossary.get('render_type')
        return mark_safe(dict(cls.CHOICES).get(render_type, ''))

    def get_render_template(self, context, instance, placeholder):
        render_type = instance.glossary.get('render_type')
        try:
            return select_template([
                '{}/breadcrumb/{}.html'.format(app_settings.APP_LABEL, render_type),
                'shop/breadcrumb/{}.html'.format(render_type),
            ])
        except TemplateDoesNotExist:
            return engines['django'].from_string('<!-- empty breadcrumb -->')

    def get_use_cache(self, context, instance, placeholder):
        try:
            app = apphook_pool.get_apphook(instance.page.application_urls)
            return app.cache_placeholders
        except (AttributeError, ImproperlyConfigured):
            return super(BreadcrumbPlugin, self).get_use_cache(context, instance, placeholder)

plugin_pool.register_plugin(BreadcrumbPlugin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.core.exceptions import PermissionDenied
from django.forms.fields import CharField
from django.forms import widgets
from django.template import engines
from django.template.loader import select_template
from django.utils.html import strip_tags, format_html
from django.utils.module_loading import import_string
from django.utils.safestring import mark_safe
from django.utils.text import Truncator
from django.utils.translation import ugettext_lazy as _, pgettext_lazy
try:
    from html.parser import HTMLParser  # py3
except ImportError:
    from HTMLParser import HTMLParser  # py2
from cms.plugin_pool import plugin_pool

from djangocms_text_ckeditor.widgets import TextEditorWidget
from djangocms_text_ckeditor.utils import plugin_tags_to_user_html
from djangocms_text_ckeditor.cms_plugins import TextPlugin

from cmsplugin_cascade.fields import GlossaryField
from cmsplugin_cascade.strides import strides_plugin_map, strides_element_map, TextStridePlugin, TextStrideElement
from cmsplugin_cascade.link.cms_plugins import TextLinkPlugin
from cmsplugin_cascade.link.forms import LinkForm, TextLinkFormMixin
from cmsplugin_cascade.link.plugin_base import LinkElementMixin
from cmsplugin_cascade.plugin_base import TransparentContainer
from cmsplugin_cascade.bootstrap3.buttons import BootstrapButtonMixin

from shop.conf import app_settings
from shop.forms.checkout import AcceptConditionForm
from shop.models.cart import CartModel
from shop.modifiers.pool import cart_modifiers_pool
from .plugin_base import ShopPluginBase, ShopButtonPluginBase, DialogFormPluginBase


class ProceedButtonForm(TextLinkFormMixin, LinkForm):
    link_content = CharField(label=_("Button Content"))
    LINK_TYPE_CHOICES = (('cmspage', _("CMS Page")), ('RELOAD_PAGE', _("Reload Page")),
        ('PURCHASE_NOW', _("Purchase Now")),)


class ShopProceedButton(BootstrapButtonMixin, ShopButtonPluginBase):
    """
    This button is used to proceed from one checkout step to the next one.
    """
    name = _("Proceed Button")
    parent_classes = ('BootstrapColumnPlugin', 'ProcessStepPlugin', 'ValidateSetOfFormsPlugin')
    model_mixins = (LinkElementMixin,)
    form = ProceedButtonForm
    glossary_field_order = ('button_type', 'button_size', 'button_options', 'quick_float',
                            'icon_align', 'icon_font', 'symbol')
    ring_plugin = 'ProceedButtonPlugin'

    class Media:
        css = {'all': ('cascade/css/admin/bootstrap.min.css',
                       'cascade/css/admin/bootstrap-theme.min.css',
                       'cascade/css/admin/iconplugin.css',)}
        js = ['shop/js/admin/proceedbuttonplugin.js']

    def get_render_template(self, context, instance, placeholder):
        template_names = [
            '{}/checkout/proceed-button.html'.format(app_settings.APP_LABEL),
            'shop/checkout/proceed-button.html',
        ]
        return select_template(template_names)

    def render(self, context, instance, placeholder):
        self.super(ShopProceedButton, self).render(context, instance, placeholder)
        try:
            cart = CartModel.objects.get_from_request(context['request'])
            cart.update(context['request'])
            context['cart'] = cart
        except CartModel.DoesNotExist:
            pass
        return context

plugin_pool.register_plugin(ShopProceedButton)


class CustomerFormPluginBase(DialogFormPluginBase):
    """
    Base class for CustomerFormPlugin and GuestFormPlugin to share common methods.
    """
    template_leaf_name = 'customer-{}.html'
    cache = False

    def get_form_data(self, context, instance, placeholder):
        form_data = self.super(CustomerFormPluginBase, self).get_form_data(context, instance, placeholder)
        form_data.update(instance=context['request'].customer)
        return form_data

    def get_render_template(self, context, instance, placeholder):
        if 'error_message' in context:
            return engines['django'].from_string('<p class="text-danger">{{ error_message }}</p>')
        return self.super(CustomerFormPluginBase, self).get_render_template(context, instance, placeholder)


class CustomerFormPlugin(CustomerFormPluginBase):
    """
    Provides the form to edit specific data stored in :class:`shop.model.customer.CustomerModel`,
    if customer declared himself as registered.
    """
    name = _("Customer Form")
    form_class = 'shop.forms.checkout.CustomerForm'

    def render(self, context, instance, placeholder):
        if not context['request'].customer.is_registered():
            context['error_message'] = _("Only registered customers can access this form.")
            return context
        return self.super(CustomerFormPlugin, self).render(context, instance, placeholder)

DialogFormPluginBase.register_plugin(CustomerFormPlugin)


class GuestFormPlugin(CustomerFormPluginBase):
    """
    Provides the form to edit specific data stored in model `Customer`, if customer declared
    himself as guest.
    """
    name = _("Guest Form")
    form_class = 'shop.forms.checkout.GuestForm'

    def render(self, context, instance, placeholder):
        if not context['customer'].is_guest():
            context['error_message'] = _("Only guest customers can access this form.")
            return context
        return self.super(GuestFormPlugin, self).render(context, instance, placeholder)

DialogFormPluginBase.register_plugin(GuestFormPlugin)


class CheckoutAddressPlugin(DialogFormPluginBase):
    name = _("Checkout Address Form")
    glossary_field_order = ['address_form', 'render_type', 'allow_multiple', 'allow_use_primary', 'headline_legend']
    form_classes = ['shop.forms.checkout.ShippingAddressForm', 'shop.forms.checkout.BillingAddressForm']
    ADDRESS_CHOICES = [('shipping', _("Shipping")), ('billing', _("Billing"))]

    address_form = GlossaryField(
        widgets.RadioSelect(choices=ADDRESS_CHOICES),
        label=_("Address Form"),
        initial=ADDRESS_CHOICES[0][0]
    )

    allow_multiple = GlossaryField(
        widgets.CheckboxInput(),
        label=_("Multiple Addresses"),
        initial=False,
        help_text=_("Allow the customer to add and edit multiple addresses."),
    )

    allow_use_primary = GlossaryField(
        widgets.CheckboxInput(),
        label=_("Use primary address"),
        initial=False,
        help_text=_("Allow the customer to use the primary address, if this is the secondary form."),
    )

    def get_form_class(self, instance):
        if instance.glossary.get('address_form') == 'shipping':
            return import_string(self.form_classes[0])
        else:  # address_form == billing
            return import_string(self.form_classes[1])

    def get_address(self, cart, instance):
        if instance.glossary.get('address_form') == 'shipping':
            if cart.shipping_address:
                address = cart.shipping_address
            else:
                # fallback to another existing shipping address
                FormClass = self.get_form_class(instance)
                address = FormClass.get_model().objects.get_fallback(customer=cart.customer)
        else:  # address_form == billing
            if cart.billing_address:
                address = cart.billing_address
            else:
                # fallback to another existing billing address
                FormClass = self.get_form_class(instance)
                address = FormClass.get_model().objects.get_fallback(customer=cart.customer)
        return address

    def get_form_data(self, context, instance, placeholder):
        form_data = self.super(CheckoutAddressPlugin, self).get_form_data(context, instance, placeholder)
        if form_data['cart'] is None:
            raise PermissionDenied("Can not proceed to checkout without cart")

        address = self.get_address(form_data['cart'], instance)
        if instance.glossary.get('allow_multiple'):
            AddressModel = self.get_form_class(instance).get_model()
            addresses = AddressModel.objects.filter(customer=context['request'].customer).order_by('priority')
            form_entities = []
            for number, addr in enumerate(addresses, 1):
                form_entities.append({
                    'value': str(addr.priority),
                    'label': "{}. {}".format(number, addr.as_text().strip().replace('\n', ' – '))
                })
            form_data.update(multi_addr=True, form_entities=form_entities)
        else:
            form_data.update(multi_addr=False)

        form_data.update(
            instance=address,
            initial={'active_priority': address.priority if address else 'add'},
            allow_use_primary=instance.glossary.get('allow_use_primary', False)
        )
        return form_data

    @classmethod
    def get_identifier(cls, instance):
        identifier = super(CheckoutAddressPlugin, cls).get_identifier(instance)
        address_form = instance.glossary.get('address_form')
        address_form = dict(cls.ADDRESS_CHOICES).get(address_form, '')
        return format_html(pgettext_lazy('get_identifier', "for {} {}"), address_form, identifier)

    def get_render_template(self, context, instance, placeholder):
        addr_form = instance.glossary.get('address_form')
        if addr_form not in ['shipping', 'billing']:  # validate
            addr_form = 'shipping'
        render_type = instance.glossary.get('render_type')
        if render_type not in ['form', 'summary']:  # validate
            render_type = 'form'
        template_names = [
            '{0}/checkout/{1}-address-{2}.html'.format(app_settings.APP_LABEL, addr_form, render_type),
            'shop/checkout/{0}-address-{1}.html'.format(addr_form, render_type),
        ]
        return select_template(template_names)

DialogFormPluginBase.register_plugin(CheckoutAddressPlugin)


class PaymentMethodFormPlugin(DialogFormPluginBase):
    name = _("Payment Method Form")
    form_class = 'shop.forms.checkout.PaymentMethodForm'
    template_leaf_name = 'payment-method-{}.html'

    def get_form_data(self, context, instance, placeholder):
        form_data = self.super(PaymentMethodFormPlugin, self).get_form_data(context, instance, placeholder)
        cart = form_data.get('cart')
        if cart:
            form_data.update(initial={'payment_modifier': cart.extra.get('payment_modifier')})
        return form_data

    def render(self, context, instance, placeholder):
        self.super(PaymentMethodFormPlugin, self).render(context, instance, placeholder)
        for payment_modifier in cart_modifiers_pool.get_payment_modifiers():
            payment_modifier.update_render_context(context)
        return context

if cart_modifiers_pool.get_payment_modifiers():
    # Plugin is registered only if at least one payment modifier exists
    DialogFormPluginBase.register_plugin(PaymentMethodFormPlugin)


class ShippingMethodFormPlugin(DialogFormPluginBase):
    name = _("Shipping Method Form")
    form_class = 'shop.forms.checkout.ShippingMethodForm'
    template_leaf_name = 'shipping-method-{}.html'

    def get_form_data(self, context, instance, placeholder):
        form_data = self.super(ShippingMethodFormPlugin, self).get_form_data(context, instance, placeholder)
        cart = form_data.get('cart')
        if cart:
            form_data.update(initial={'shipping_modifier': cart.extra.get('shipping_modifier')})
        return form_data

    def render(self, context, instance, placeholder):
        self.super(ShippingMethodFormPlugin, self).render(context, instance, placeholder)
        for shipping_modifier in cart_modifiers_pool.get_shipping_modifiers():
            shipping_modifier.update_render_context(context)
        return context

if cart_modifiers_pool.get_shipping_modifiers():
    # Plugin is registered only if at least one shipping modifier exists
    DialogFormPluginBase.register_plugin(ShippingMethodFormPlugin)


class ExtraAnnotationFormPlugin(DialogFormPluginBase):
    name = _("Extra Annotation Form")
    form_class = 'shop.forms.checkout.ExtraAnnotationForm'
    template_leaf_name = 'extra-annotation-{}.html'

    def get_form_data(self, context, instance, placeholder):
        form_data = self.super(ExtraAnnotationFormPlugin, self).get_form_data(context, instance, placeholder)
        cart = form_data.get('cart')
        if cart:
            form_data.update(initial={'annotation': cart.extra.get('annotation', '')})
        return form_data

DialogFormPluginBase.register_plugin(ExtraAnnotationFormPlugin)


class AcceptConditionFormPlugin(DialogFormPluginBase):
    """
    Deprecated. Use AcceptConditionPlugin instead.
    """
    name = _("Accept Condition (deprecated)")
    form_class = 'shop.forms.checkout.AcceptConditionForm'
    template_leaf_name = 'accept-condition.html'
    html_parser = HTMLParser()
    change_form_template = 'cascade/admin/text_plugin_change_form.html'

    @classmethod
    def get_identifier(cls, instance):
        html_content = cls.html_parser.unescape(instance.glossary.get('html_content', ''))
        html_content = strip_tags(html_content)
        html_content = Truncator(html_content).words(3, truncate=' ...')
        return mark_safe(html_content)

    def get_form(self, request, obj=None, **kwargs):
        if obj:
            html_content = self.html_parser.unescape(obj.glossary.get('html_content', ''))
            obj.glossary.update(html_content=html_content)
            text_editor_widget = TextEditorWidget(installed_plugins=[TextLinkPlugin], pk=obj.pk,
                                           placeholder=obj.placeholder, plugin_language=obj.language)
            kwargs['glossary_fields'] = (
                GlossaryField(text_editor_widget, label=_("HTML content"), name='html_content'),
            )
        return super(AcceptConditionFormPlugin, self).get_form(request, obj, **kwargs)

    def render(self, context, instance, placeholder):
        self.super(AcceptConditionFormPlugin, self).render(context, instance, placeholder)
        accept_condition_form = context['accept_condition_form.plugin_{}'.format(instance.id)]
        html_content = self.html_parser.unescape(instance.glossary.get('html_content', ''))
        html_content = plugin_tags_to_user_html(html_content, context)
        # transfer the stored HTML content into the widget's label
        accept_condition_form['accept'].field.widget.choice_label = mark_safe(html_content)
        context['accept_condition_form'] = accept_condition_form
        return context

# DialogFormPluginBase.register_plugin(AcceptConditionFormPlugin)

class AcceptConditionMixin(object):
    render_template = 'shop/checkout/accept-condition.html'
    FormClass = AcceptConditionForm

    def render(self, context, instance, placeholder):
        """
        Return the context to render a checkbox used to accept the terms and conditions
        """
        request = context['request']
        try:
            cart = CartModel.objects.get_from_request(request)
            cart.update(request)
        except CartModel.DoesNotExist:
            cart = None
        request._plugin_order = getattr(request, '_plugin_order', 0) + 1
        form_data = {'cart': cart, 'initial': dict(plugin_id=instance.pk, plugin_order=request._plugin_order)}
        bound_form = self.FormClass(**form_data)
        context[bound_form.form_name] = bound_form
        super(AcceptConditionMixin, self).render(context, instance, placeholder)
        accept_condition_form = context['accept_condition_form.plugin_{}'.format(instance.pk)]
        # transfer the stored HTML content into the widget's label
        accept_condition_form['accept'].field.widget.choice_label = mark_safe(context['body'])
        context['accept_condition_form'] = accept_condition_form
        return context


class AcceptConditionPlugin(AcceptConditionMixin, TextPlugin):
    name = _("Accept Condition")
    module = "Shop"

    def get_admin_url_name(self, name):
        model_name = 'acceptcondition'
        url_name = "%s_%s_%s" % ('shop', model_name, name)
        return url_name


class AcceptConditionMinion(AcceptConditionMixin, TextStridePlugin):
    pass

plugin_pool.register_plugin(AcceptConditionPlugin)
strides_plugin_map['AcceptConditionPlugin'] = AcceptConditionMinion
strides_element_map['AcceptConditionPlugin'] = TextStrideElement


class RequiredFormFieldsPlugin(ShopPluginBase):
    """
    This plugin renders a short text message, emphasizing that fields with a star are required.
    """
    name = _("Required Form Fields")
    template_leaf_name = 'required-form-fields.html'
    parent_classes = ('BootstrapColumnPlugin',)

    def get_render_template(self, context, instance, placeholder):
        template_names = [
            '{0}/checkout/{1}'.format(app_settings.APP_LABEL, self.template_leaf_name),
            'shop/checkout/{}'.format(self.template_leaf_name),
        ]
        return select_template(template_names)

plugin_pool.register_plugin(RequiredFormFieldsPlugin)


class ValidateSetOfFormsPlugin(TransparentContainer, ShopPluginBase):
    """
    This plugin wraps arbitrary forms into the Angular directive shopFormsSet.
    This is required to validate all forms, so that a proceed button is disabled otherwise.
    """
    name = _("Validate Set of Forms")
    allow_children = True
    alien_child_classes = True

    def get_render_template(self, context, instance, placeholder):
        return select_template([
            '{}/checkout/forms-set.html'.format(app_settings.APP_LABEL),
            'shop/checkout/forms-set.html',
        ])

plugin_pool.register_plugin(ValidateSetOfFormsPlugin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from collections import OrderedDict
from datetime import datetime

from django import template
from django.conf import settings
from django.template.loader import select_template
from django.utils import formats
from django.utils.html import force_text
from django.utils.safestring import mark_safe
from django.utils.dateformat import format, time_format

from classytags.helpers import InclusionTag

from shop.conf import app_settings
from shop.models.cart import CartModel
from shop.rest.money import JSONRenderer

register = template.Library()


class CartIcon(InclusionTag):
    """
    Inclusion tag for displaying cart summary.
    """
    def get_template(self, context, **kwargs):
        template = select_template([
            '{}/templatetags/cart-icon.html'.format(app_settings.APP_LABEL),
            'shop/templatetags/cart-icon.html',
        ])
        return template.template.name

    def get_context(self, context):
        request = context['request']
        try:
            cart = CartModel.objects.get_from_request(request)
            cart.update(request)
            context['cart'] = cart
        except CartModel.DoesNotExist:
            pass
        return context
register.tag(CartIcon)


def from_iso8601(value):
    try:
        return datetime.strptime(value, "%Y-%m-%dT%H:%M:%S.%fZ")
    except ValueError:
        return datetime.strptime(value, "%Y-%m-%dT%H:%M:%SZ")


@register.filter(expects_localtime=True, is_safe=False)
def date(value, arg=None):
    """
    Alternative implementation to the built-in `date` template filter which also accepts the
    date string in iso-8601 as passed in by the REST serializers.
    """
    if value in (None, ''):
        return ''
    if not isinstance(value, datetime):
        value = from_iso8601(value)
    if arg is None:
        arg = settings.DATE_FORMAT
    try:
        return formats.date_format(value, arg)
    except AttributeError:
        try:
            return format(value, arg)
        except AttributeError:
            return ''


@register.filter(expects_localtime=True, is_safe=False)
def time(value, arg=None):
    """
    Alternative implementation to the built-in `time` template filter which also accepts the
    date string in iso-8601 as passed in by the REST serializers.
    """
    if value in (None, ''):
        return ''
    if not isinstance(value, datetime):
        value = from_iso8601(value)
    if arg is None:
        arg = settings.TIME_FORMAT
    try:
        return formats.time_format(value, arg)
    except AttributeError:
        try:
            return time_format(value, arg)
        except AttributeError:
            return ''


@register.filter
def rest_json(value, arg=None):
    """
    Renders a `ReturnDict` as used by the REST framework into a safe JSON string.
    """
    if isinstance(value, (dict, OrderedDict, list, tuple)):
        data = JSONRenderer().render(value)
    elif not value:
        data = '{}'
    else:
        msg = "Given value must be of type dict, OrderedDict, list or tuple but it is {}."
        raise ValueError(msg.format(value.__class__.__name__))
    return mark_safe(force_text(data))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals


class ShippingProvider(object):
    """
    Base class for all Shipping Providers.
    """
    @property
    def namespace(self):
        """
        Use a unique namespace for this shipping provider. It is used to keep state over how each
        item was shipped to the customer.
        """
        msg = "The attribute `namespace` must be implemented by the class `{}`"
        raise NotImplementedError(msg.format(self.__class__.__name__))

    def get_urls(self):
        """
        Return a list of URL patterns for external communication with the shipping service provider.
        """
        return []



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models
from django.utils.translation import ugettext_lazy as _

from shop.models.address import BaseShippingAddress, BaseBillingAddress, CountryField


class AddressModelMixin(models.Model):
    name = models.CharField(_("Full name"), max_length=1024)
    address1 = models.CharField(_("Address line 1"), max_length=1024)
    address2 = models.CharField(_("Address line 2"), max_length=1024, blank=True, null=True)
    zip_code = models.CharField(_("ZIP / Postal code"), max_length=12)
    city = models.CharField(_("City"), max_length=1024)
    country = CountryField(_("Country"))

    class Meta:
        abstract = True


class ShippingAddress(BaseShippingAddress, AddressModelMixin):
    class Meta:
        verbose_name = _("Shipping Address")
        verbose_name_plural = _("Shipping Addresses")


class BillingAddress(BaseBillingAddress, AddressModelMixin):
    class Meta:
        verbose_name = _("Billing Address")
        verbose_name_plural = _("Billing Addresses")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf.urls import include, url

from . import rest_api
from . import auth
from . import payment


urlpatterns = [
    url(r'^api/', include(rest_api)),
    url(r'^auth/', include(auth)),
    url(r'^payment/', include(payment)),
]



# -*- coding: utf-8 -*-
"""
In django-SHOP, a commodity is considered a very basic product without any attributes, which can
be used on a generic CMS page to describe anything.
"""
from __future__ import unicode_literals

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from cms.models.fields import PlaceholderField
from filer.fields import image
from djangocms_text_ckeditor.fields import HTMLField
from polymorphic.query import PolymorphicQuerySet

from shop.conf import app_settings
from shop.models.product import BaseProduct, BaseProductManager, CMSPageReferenceMixin
from shop.models.defaults.mapping import ProductPage
from shop.money.fields import MoneyField


if settings.USE_I18N:
    if 'parler' not in settings.INSTALLED_APPS:
        raise ImproperlyConfigured("Requires `django-parler`, if configured as multilingual project")

    from parler.managers import TranslatableManager, TranslatableQuerySet
    from parler.models import TranslatableModelMixin, TranslatedFieldsModel
    from parler.fields import TranslatedField


    class ProductQuerySet(TranslatableQuerySet, PolymorphicQuerySet):
        pass


    class ProductManager(BaseProductManager, TranslatableManager):
        queryset_class = ProductQuerySet


    @python_2_unicode_compatible
    class Commodity(CMSPageReferenceMixin, TranslatableModelMixin, BaseProduct):
        """
        Generic Product Commodity to be used whenever the merchant does not require product specific
        attributes and just required a placeholder field to add arbitrary data.
        """
        # common product fields
        product_code = models.CharField(_("Product code"), max_length=255, unique=True)
        unit_price = MoneyField(_("Unit price"), decimal_places=3,
                                help_text=_("Net price for this product"))

        # controlling the catalog
        order = models.PositiveIntegerField(verbose_name=_("Sort by"), db_index=True)
        cms_pages = models.ManyToManyField('cms.Page', through=ProductPage,
            help_text=_("Choose list view this product shall appear on."))
        sample_image = image.FilerImageField(verbose_name=_("Sample Image"), blank=True, null=True,
            help_text=_("Sample image used in the catalog's list view."))
        show_breadcrumb = models.BooleanField(_("Show Breadcrumb"), default=True,
            help_text=_("Shall the detail page show the product's breadcrumb."))
        placeholder = PlaceholderField("Commodity Details")

        # translatable fields for the catalog's list- and detail views
        product_name = TranslatedField()
        slug = TranslatedField()
        caption = TranslatedField()

        # filter expression used to search for a product item using the Select2 widget
        lookup_fields = ('product_code__startswith', 'product_name__icontains',)

        objects = ProductManager()

        class Meta:
            app_label = app_settings.APP_LABEL
            ordering = ('order',)
            verbose_name = _("Commodity")
            verbose_name_plural = _("Commodities")

        def __str__(self):
            return self.product_code

        def get_price(self, request):
            return self.unit_price


    class CommodityTranslation(TranslatedFieldsModel):
        master = models.ForeignKey(Commodity, related_name='translations', null=True)
        product_name = models.CharField(max_length=255, verbose_name=_("Product Name"))
        slug = models.SlugField(verbose_name=_("Slug"))
        caption = HTMLField(verbose_name=_("Caption"), blank=True, null=True,
                            help_text=_("Short description for the catalog list view."))

        class Meta:
            app_label = app_settings.APP_LABEL
            unique_together = [('language_code', 'master')]

else:

    @python_2_unicode_compatible
    class Commodity(CMSPageReferenceMixin, BaseProduct):
        """
        Generic Product Commodity to be used whenever the merchant does not require product specific
        attributes and just required a placeholder field to add arbitrary data.
        """
        # common product fields
        product_name = models.CharField(max_length=255, verbose_name=_("Product Name"))
        product_code = models.CharField(_("Product code"), max_length=255, unique=True)
        unit_price = MoneyField(_("Unit price"), decimal_places=3,
                                help_text=_("Net price for this product"))

        # controlling the catalog
        order = models.PositiveIntegerField(verbose_name=_("Sort by"), db_index=True)
        cms_pages = models.ManyToManyField('cms.Page', through=ProductPage,
            help_text=_("Choose list view this product shall appear on."))
        sample_image = image.FilerImageField(verbose_name=_("Sample Image"), blank=True, null=True,
            help_text=_("Sample image used in the catalog's list view."))
        show_breadcrumb = models.BooleanField(_("Show Breadcrumb"), default=True,
            help_text=_("Shall the detail page show the product's breadcrumb."))
        placeholder = PlaceholderField("Commodity Details")

        # common fields for the catalog's list- and detail views
        slug = models.SlugField(verbose_name=_("Slug"))
        caption = HTMLField(verbose_name=_("Caption"), blank=True, null=True,
                            help_text=_("Short description for the catalog list view."))

        # filter expression used to search for a product item using the Select2 widget
        lookup_fields = ('product_code__startswith', 'product_name__icontains',)

        objects = BaseProductManager()

        class Meta:
            app_label = app_settings.APP_LABEL
            ordering = ('order',)
            verbose_name = _("Commodity")
            verbose_name_plural = _("Commodities")

        def __str__(self):
            return self.product_code

        def get_price(self, request):
            return self.unit_price

        def save(self, *args, **kwargs):
            if self.order is None:
                aggr = self._meta.model.objects.aggregate(max=models.Max('order'))
                self.order = 1 if aggr['max'] is None else aggr['max'] + 1
            return super(Commodity, self).save(*args, **kwargs)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-09-10 11:14
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import shop.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('shop', '0007_notification_recipient'),
    ]

    operations = [
        migrations.AlterField(
            model_name='notification',
            name='notify',
            field=shop.models.fields.ChoiceEnumField(verbose_name='Whom to notify'),
        ),
        migrations.AlterField(
            model_name='notification',
            name='recipient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='Recipient'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.core.exceptions import ImproperlyConfigured
from django.utils.translation import ugettext_lazy as _

from django_fsm import transition, RETURN_VALUE

from shop.models.order import BaseOrder, OrderModel
from .base import PaymentProvider


class ForwardFundPayment(PaymentProvider):
    """
    Provides a simple prepayment payment provider.
    """
    namespace = 'forward-fund-payment'

    def get_payment_request(self, cart, request):
        order = OrderModel.objects.create_from_cart(cart, request)
        order.populate_from_cart(cart, request)
        if order.total == 0:
            order.no_payment_required()
        else:
            order.awaiting_payment()
        order.save()
        thank_you_url = OrderModel.objects.get_latest_url()
        return '$window.location.href="{}";'.format(thank_you_url)


class ManualPaymentWorkflowMixin(object):
    """
    Add this class to `settings.SHOP_ORDER_WORKFLOWS` to mix it into your `OrderModel`.
    It adds all the methods required for state transitions, when used with the
    `ForwardFundPayment` provider from above.
    """
    TRANSITION_TARGETS = {
        'awaiting_payment': _("Awaiting a forward fund payment"),
        'prepayment_deposited': _("Prepayment deposited"),
        'no_payment_required': _("No Payment Required"),
    }

    def __init__(self, *args, **kwargs):
        if not isinstance(self, BaseOrder):
            raise ImproperlyConfigured("class 'ManualPaymentWorkflowMixin' is not of type 'BaseOrder'")

        CancelOrderWorkflowMixin.CANCELABLE_SOURCES.update(['awaiting_payment', 'prepayment_deposited',
                                                            'no_payment_required'])
        super(ManualPaymentWorkflowMixin, self).__init__(*args, **kwargs)

    def is_fully_paid(self):
        return super(ManualPaymentWorkflowMixin, self).is_fully_paid()

    @transition(field='status', source=['created'], target='no_payment_required')
    def no_payment_required(self):
        """
        Signals that an Order can proceed directly, by confirming a payment of value zero.
        """

    @transition(field='status', source=['created'], target='awaiting_payment')
    def awaiting_payment(self):
        """
        Signals that the current Order awaits a payment.
        Invoked by ForwardFundPayment.get_payment_request.
        """

    def deposited_too_little(self):
        return self.amount_paid > 0 and self.amount_paid < self.total

    @transition(field='status', source=['awaiting_payment'], target='awaiting_payment',
                conditions=[deposited_too_little], custom=dict(admin=True, button_name=_("Deposited too little")))
    def prepayment_partially_deposited(self):
        """
        Signals that the current Order received a payment, which was not enough.
        """

    @transition(field='status', source=['awaiting_payment'], target='prepayment_deposited',
                conditions=[is_fully_paid], custom=dict(admin=True, button_name=_("Mark as Paid")))
    def prepayment_fully_deposited(self):
        """
        Signals that the current Order received a payment, which fully covers the requested sum.
        """

    @transition(field='status', source=['prepayment_deposited', 'no_payment_required'],
                custom=dict(auto=True))
    def acknowledge_prepayment(self):
        """
        Acknowledge the payment. This method is invoked automatically.
        """
        self.acknowledge_payment()

    @transition(field='status', source='refund_payment', target=RETURN_VALUE('refund_payment', 'order_canceled'),
                custom=dict(admin=True, button_name=_("Mark as Refunded")))
    def payment_refunded(self):
        """
        Signals that the payment for this Order has been refunded manually.
        """
        return 'refund_payment' if self.amount_paid else 'order_canceled'


class CancelOrderWorkflowMixin(object):
    """
    Add this class to `settings.SHOP_ORDER_WORKFLOWS` to mix it into your `OrderModel`.
    It adds all the methods required for state transitions, to cancel an order.
    """
    CANCELABLE_SOURCES = {'new', 'created', 'payment_confirmed', 'payment_declined'}
    TRANSITION_TARGETS = {
        'refund_payment': _("Refund payment"),
        'order_canceled': _("Order Canceled"),
    }

    def cancelable(self):
        return self.status in self.CANCELABLE_SOURCES

    @transition(field='status', target=RETURN_VALUE(*TRANSITION_TARGETS.keys()),
                conditions=[cancelable], custom=dict(admin=True, button_name=_("Cancel Order")))
    def cancel_order(self):
        """
        Signals that an Order shall be canceled.
        """
        if self.amount_paid:
            self.refund_payment()
        return 'refund_payment' if self.amount_paid else 'order_canceled'



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models
from django.utils.translation import ugettext_lazy as _
from shop.models.delivery import BaseDelivery, BaseDeliveryItem


class Delivery(BaseDelivery):
    """Default materialized model for OrderShipping"""


class DeliveryItem(BaseDeliveryItem):
    """Default materialized model for ShippedItem"""
    quantity = models.IntegerField(_("Delivered quantity"), default=0)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.conf.urls import url, include
from rest_framework import routers
from shop.views.cart import CartViewSet, WatchViewSet
from shop.views.checkout import CheckoutViewSet
from shop.views.catalog import ProductSelectView

router = routers.DefaultRouter()  # TODO: try with trailing_slash=False
router.register(r'cart', CartViewSet, base_name='cart')
router.register(r'watch', WatchViewSet, base_name='watch')
router.register(r'checkout', CheckoutViewSet, base_name='checkout')

urlpatterns = [
    url(r'^select_product/?$', ProductSelectView.as_view(), name='select-product'),
    url(r'^', include(router.urls)),
]



XXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXX

XXXXXXXXXXXXXXXXXXXXXXX
XXXXXX



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from shop.admin.order import BaseOrderAdmin


class OrderAdmin(BaseOrderAdmin):
    """
    Admin class to be used for Order model :class:`shop.models.defaults.order`
    """
    def get_fields(self, request, obj=None):
        fields = list(super(OrderAdmin, self).get_fields(request, obj))
        fields.extend(['shipping_address_text', 'billing_address_text'])
        return fields

    def get_readonly_fields(self, request, obj=None):
        readonly_fields = list(super(OrderAdmin, self).get_readonly_fields(request, obj))
        readonly_fields.extend(['shipping_address_text', 'billing_address_text'])
        return readonly_fields

    def get_search_fields(self, request):
        search_fields = list(super(OrderAdmin, self).get_search_fields(request))
        search_fields.extend(['number', 'shipping_address_text', 'billing_address_text'])
        return search_fields



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django import forms
from django.forms import widgets
from django.core.exceptions import ValidationError
from django.template import engines
from django.template.loader import select_template
from django.utils.translation import ugettext_lazy as _

from cms.plugin_pool import plugin_pool
from cmsplugin_cascade.bootstrap3.buttons import BootstrapButtonMixin
from cmsplugin_cascade.fields import GlossaryField
from cmsplugin_cascade.plugin_base import TransparentWrapper

from djng.forms import fields, NgModelFormMixin
from djng.styling.bootstrap3.forms import Bootstrap3Form

from shop.conf import app_settings
from .extensions import ShopExtendableMixin, LeftRightExtensionMixin
from .plugin_base import ShopPluginBase


class ShopOrderViewsForm(forms.ModelForm):
    def clean(self):
        cleaned_data = super(ShopOrderViewsForm, self).clean()
        if self.instance.page and self.instance.page.application_urls != 'OrderApp':
            msg = "This plugin only makes sense if used on a CMS page with an application of type 'OrderApp'."
            raise ValidationError(msg)
        return cleaned_data


class ShopOrderViewsPlugin(LeftRightExtensionMixin, TransparentWrapper, ShopPluginBase):
    name = _("Order Views")
    require_parent = True
    parent_classes = ('BootstrapColumnPlugin',)
    allow_children = True
    model_mixins = (ShopExtendableMixin,)
    form = ShopOrderViewsForm
    cache = False

    def get_render_template(self, context, instance, placeholder):
        many = context.get('many')
        if many is True:
            # render Order List View
            return select_template([
                '{}/order/list.html'.format(app_settings.APP_LABEL),
                'shop/order/list.html',
            ])
        if many is False:
            # render Order Detail View
            return select_template([
                '{}/order/detail.html'.format(app_settings.APP_LABEL),
                'shop/order/detail.html',
            ])
        # can happen, if this plugin is abused outside of an OrderView
        return engines['django'].from_string('<pre class="bg-danger">This {} plugin is used on a CMS page without an application of type "View Order".</pre>'.format(self.name))

plugin_pool.register_plugin(ShopOrderViewsPlugin)


class OrderButtonForm(ShopOrderViewsForm):
    button_content = fields.CharField(required=False, label=_("Button Content"),
                                      widget=widgets.TextInput())

    def __init__(self, raw_data=None, *args, **kwargs):
        instance = kwargs.get('instance')
        if instance:
            initial = {'button_content': instance.glossary.get('button_content') }
            kwargs.update(initial=initial)
        super(OrderButtonForm, self).__init__(raw_data, *args, **kwargs)

    def clean(self):
        cleaned_data = super(OrderButtonForm, self).clean()
        if self.is_valid():
            cleaned_data['glossary']['button_content'] = cleaned_data['button_content']
        return cleaned_data


class OrderButtonBase(BootstrapButtonMixin, ShopPluginBase):
    parent_classes = ['ShopOrderViewsPlugin']
    form = OrderButtonForm
    fields = ['button_content', 'glossary']
    glossary_field_order = ['button_type', 'button_size', 'button_options', 'quick_float',
                            'icon_align', 'icon_font', 'symbol']

    class Media:
        css = {'all': ('cascade/css/admin/bootstrap.min.css', 'cascade/css/admin/bootstrap-theme.min.css',)}

    @classmethod
    def get_identifier(cls, instance):
        return instance.glossary.get('button_content', '')

    def render(self, context, instance, placeholder):
        context = super(OrderButtonBase, self).render(context, instance, placeholder)
        context.update({
            'button_label': instance.glossary.get('button_content', '')
        })
        return context


class ShopReorderButtonPlugin(OrderButtonBase):
    name = _("Reorder Button")

    def get_render_template(self, context, instance, placeholder):
        template_names = [
            '{}/order/reorder-button.html'.format(app_settings.APP_LABEL),
            'shop/order/reorder-button.html',
        ]
        return select_template(template_names)

plugin_pool.register_plugin(ShopReorderButtonPlugin)


class ShopCancelOrderButtonPlugin(OrderButtonBase):
    name = _("Cancel Order Button")

    def get_render_template(self, context, instance, placeholder):
        template_names = [
            '{}/order/cancel-button.html'.format(app_settings.APP_LABEL),
            'shop/order/cancel-button.html',
        ]
        return select_template(template_names)

plugin_pool.register_plugin(ShopCancelOrderButtonPlugin)


class AddendumForm(NgModelFormMixin, Bootstrap3Form):
    annotation = fields.CharField(
        label=_("Supplementary annotation for this Order"),
        widget=widgets.Textarea(attrs={'rows': 2}),
    )


class ShopOrderAddendumFormPlugin(OrderButtonBase):
    name = _("Order Addendum Form")

    show_history = GlossaryField(
         widgets.CheckboxInput(),
         label=_("Show History"),
         initial=True,
         help_text=_("Show historical annotations.")
    )

    def get_render_template(self, context, instance, placeholder):
        template_names = [
            '{}/order/addenum-form.html'.format(app_settings.APP_LABEL),
            'shop/order/addenum-form.html',
        ]
        return select_template(template_names)

    def render(self, context, instance, placeholder):
        context = super(ShopOrderAddendumFormPlugin, self).render(context, instance, placeholder)
        context.update({
            'addenum_form': AddendumForm(),
            'show_history': instance.glossary.get('show_history', True),
        })
        return context

plugin_pool.register_plugin(ShopOrderAddendumFormPlugin)



# -*- coding: utf-8 -*-
"""
The models in directory default have been added as a pure convenience and for demonstration
purpose. Whenever there is a need to use a modified version, copy one of these models into
the projects model directory and adopt it to your needs. Otherwise just import the model
into your own models.py file without using it. The latter is important to materialize the
model.

Each model is declared in its own file. This is to prevent model validation errors on related
fields if a file containing this definition is imported without using the model.
"""



XXXXXXXXXXX
XXXXXXXXXXX
XXXXXXXXX
XXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXX
XXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXX


# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.utils.formats import number_format
from rest_framework import renderers
from rest_framework import serializers
from rest_framework.utils import encoders
from shop.money import AbstractMoney


class JSONEncoder(encoders.JSONEncoder):
    """JSONEncoder subclass that knows how to encode Money."""

    def default(self, obj):
        if isinstance(obj, AbstractMoney):
            return number_format(obj)
        return super(JSONEncoder, self).default(obj)


class JSONRenderer(renderers.JSONRenderer):
    encoder_class = JSONEncoder


class MoneyField(serializers.Field):
    """Money objects are serialized into their readable notation."""

    def __init__(self, *args, **kwargs):
        kwargs.update(read_only=True)
        super(MoneyField, self).__init__(*args, **kwargs)

    def to_representation(self, obj):
        return number_format(obj)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-08-18 13:20
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import shop.models.fields
from shop.models.notification import Notify


def forwards(apps, schema_editor):
    if schema_editor.connection.alias != 'default':
        return
    Notification = apps.get_model('shop', 'Notification')
    for row in Notification.objects.all():
        if row.mail_to is None:
            row.notify = Notify.NOBODY
        if row.mail_to == 0:
            row.mail_to = None
            row.notify = Notify.CUSTOMER
        row.save(update_fields=['mail_to', 'notify'])


def backwards(apps, schema_editor):
    if schema_editor.connection.alias != 'default':
        return
    Notification = apps.get_model('shop', 'Notification')
    for row in Notification.objects.all():
        if row.notify is Notify.NOBODY:
            row.mail_to = None
        if row.notify is Notify.CUSTOMER:
            row.mail_to = 0
        row.save(update_fields=['mail_to'])


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('shop', '0006_rename_to_catalog'),
    ]

    operations = [
        migrations.AddField(
            model_name='notification',
            name='notify',
            field=shop.models.fields.ChoiceEnumField(verbose_name='Whom to notify', default=Notify.NOBODY),
        ),
        migrations.RunPython(forwards, reverse_code=backwards),
        migrations.RenameField(
            model_name='notification',
            old_name='mail_to',
            new_name='recipient',
        ),
        migrations.AlterField(
            model_name='notification',
            name='recipient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='notifications', to=settings.AUTH_USER_MODEL, verbose_name='Recipient'),
        ),
        migrations.AlterModelOptions(
            name='notification',
            options={'ordering': ['transition_target', 'recipient_id'], 'verbose_name': 'Notification',
                     'verbose_name_plural': 'Notifications'},
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import warnings

from django import forms
from django.core.cache import cache
from django.core.exceptions import ImproperlyConfigured
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from adminsortable2.admin import SortableInlineAdminMixin

from cms.models import Page

from shop.models.related import ProductPageModel, ProductImageModel


class ProductImageInline(SortableInlineAdminMixin, admin.StackedInline):
    model = ProductImageModel
    extra = 1
    ordering = ('order',)


def _find_catalog_list_apphook():
    from shop.cms_apphooks import CatalogListCMSApp
    from cms.apphook_pool import apphook_pool

    for name, app in apphook_pool.apps.items():
        if isinstance(app, CatalogListCMSApp):
            return name
    else:
        raise ImproperlyConfigured("You must register a CMS apphook of type `CatalogListCMSApp`.")


class CMSPageAsCategoryMixin(object):
    """
    Add this mixin class to the ModelAdmin class for products wishing to be assigned to djangoCMS
    pages when used as categories.
    """
    def __init__(self, *args, **kwargs):
        super(CMSPageAsCategoryMixin, self).__init__(*args, **kwargs)
        if not hasattr(self.model, 'cms_pages'):
            raise ImproperlyConfigured("Product model requires a field named `cms_pages`")

    def get_fieldsets(self, request, obj=None):
        fieldsets = list(super(CMSPageAsCategoryMixin, self).get_fieldsets(request, obj=obj))
        fieldsets.append((_("Categories"), {'fields': ('cms_pages',)}),)
        return fieldsets

    def get_fields(self, request, obj=None):
        # In ``get_fieldsets()``, ``cms_pages`` is added, so remove it from ``fields`` to
        # avoid showing it twice.
        fields = list(super(CMSPageAsCategoryMixin, self).get_fields(request, obj))
        try:
            fields.remove('cms_pages')
        except ValueError:
            pass
        return fields

    def formfield_for_manytomany(self, db_field, request, **kwargs):
        if db_field.name == 'cms_pages':
            # restrict many-to-many field for cms_pages to ProductApp only
            limit_choices_to = {
                'publisher_is_draft': False,
                'application_urls': getattr(self, 'limit_to_cmsapp', _find_catalog_list_apphook()),
            }
            queryset = Page.objects.filter(**limit_choices_to)
            widget = admin.widgets.FilteredSelectMultiple(_("CMS Pages"), False)
            required = not db_field.blank
            field = forms.ModelMultipleChoiceField(queryset=queryset, widget=widget, required=required)
            return field
        return super(CMSPageAsCategoryMixin, self).formfield_for_manytomany(db_field, request, **kwargs)

    def save_related(self, request, form, formsets, change):
        old_cms_pages = form.instance.cms_pages.all()
        new_cms_pages = form.cleaned_data.pop('cms_pages')

        # remove old
        for page in old_cms_pages:
            if page not in new_cms_pages:
                for pp in ProductPageModel.objects.filter(product=form.instance, page=page):
                    pp.delete()

        # add new
        for page in new_cms_pages:
            if page not in old_cms_pages:
                ProductPageModel.objects.create(product=form.instance, page=page)

        return super(CMSPageAsCategoryMixin, self).save_related(request, form, formsets, change)


class InvalidateProductCacheMixin(object):
    def __init__(self, *args, **kwargs):
        if not hasattr(cache, 'delete_pattern'):
            warnings.warn("\n"
                "Your caching backend does not support deletion by key patterns.\n"
                "Please use 'django-redis-cache', or wait until the product's HTML\n"
                "snippet cache expires by itself.")
        super(InvalidateProductCacheMixin, self).__init__(*args, **kwargs)

    def save_model(self, request, product, form, change):
        if change:
            self.invalidate_cache(product)
        super(InvalidateProductCacheMixin, self).save_model(request, product, form, change)

    def invalidate_cache(self, product):
        """
        The method ``ProductCommonSerializer.render_html`` caches the rendered HTML snippets.
        Invalidate them after changing something in the product.
        """
        try:
            cache.delete_pattern('product:{}|*'.format(product.id))
        except AttributeError:
            pass


class CMSPageFilter(admin.SimpleListFilter):
    title = _("Category")
    parameter_name = 'category'

    def lookups(self, request, model_admin):
        limit_choices_to = {
            'publisher_is_draft': False,
            'application_urls': getattr(self, 'limit_to_cmsapp', _find_catalog_list_apphook())
        }
        queryset = Page.objects.filter(**limit_choices_to)
        return [(page.id, page.get_title()) for page in queryset]

    def queryset(self, request, queryset):
        if self.value():
            return queryset.filter(cms_pages__id=self.value())



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib.admin import StackedInline
from django.forms import widgets
from django.forms.models import ModelForm
from django.template.loader import select_template
from django.utils.translation import ugettext_lazy as _, ugettext

from cms.plugin_pool import plugin_pool
from cms.utils.compat.dj import is_installed
from cmsplugin_cascade.mixins import WithSortableInlineElementsMixin
from cmsplugin_cascade.models import SortableInlineCascadeElement
from cmsplugin_cascade.fields import GlossaryField

from shop.conf import app_settings
from shop.models.product import ProductModel
from .plugin_base import ShopPluginBase, ProductSelectField

if is_installed('adminsortable2'):
    from adminsortable2.admin import SortableInlineAdminMixin
else:
    SortableInlineAdminMixin = type(str('SortableInlineAdminMixin'), (object,), {})


class ShopCatalogPlugin(ShopPluginBase):
    name = _("Catalog List View")
    require_parent = True
    parent_classes = ('BootstrapColumnPlugin', 'SimpleWrapperPlugin',)
    cache = False

    infinite_scroll = GlossaryField(
        widgets.CheckboxInput(),
        label=_("Infinite Scroll"),
        initial=True,
        help_text=_("Shall the product list view scroll infinitely?"),
    )

    def get_render_template(self, context, instance, placeholder):
        return select_template([
            '{}/catalog/product-list.html'.format(app_settings.APP_LABEL),
            'shop/catalog/product-list.html',
        ])

    def render(self, context, instance, placeholder):
        context['infinite_scroll'] = bool(instance.glossary.get('infinite_scroll', True))
        return context

    @classmethod
    def get_identifier(cls, obj):
        if obj.glossary.get('infinite_scroll', True):
            return ugettext("Infinite Scroll")
        return ugettext("Manual Pagination")

plugin_pool.register_plugin(ShopCatalogPlugin)


class ShopAddToCartPlugin(ShopPluginBase):
    name = _("Add Product to Cart")
    require_parent = True
    parent_classes = ('BootstrapColumnPlugin',)
    cache = False

    def get_render_template(self, context, instance, placeholder):
        return select_template([
            '{}/catalog/product-add2cart.html'.format(app_settings.APP_LABEL),
            'shop/catalog/product-add2cart.html',
        ])

plugin_pool.register_plugin(ShopAddToCartPlugin)


class ProductGalleryForm(ModelForm):
    product = ProductSelectField(required=False, label=_("Related Product"),
        help_text=_("Choose related product"))

    class Meta:
        exclude = ('glossary',)

    def __init__(self, *args, **kwargs):
        try:
            initial = dict(kwargs['instance'].glossary)
        except (KeyError, AttributeError):
            initial = {}
        initial.update(kwargs.pop('initial', {}))
        try:
            self.base_fields['product'].initial = initial['product']['pk']
        except KeyError:
            self.base_fields['product'].initial = None
        if not is_installed('adminsortable2'):
            self.base_fields['order'].widget = widgets.HiddenInput()
            self.base_fields['order'].initial = 0
        super(ProductGalleryForm, self).__init__(*args, **kwargs)

    def clean(self):
        cleaned_data = super(ProductGalleryForm, self).clean()
        if self.is_valid():
            product_pk = self.cleaned_data.pop('product', None)
            if product_pk:
                product_data = {'pk': product_pk}
                self.instance.glossary.update(product=product_data)
            else:
                self.instance.glossary.pop('image', None)
        return cleaned_data


class ProductGalleryInline(SortableInlineAdminMixin, StackedInline):
    model = SortableInlineCascadeElement
    raw_id_fields = ('product',)
    form = ProductGalleryForm
    extra = 5
    ordering = ('order',)
    verbose_name = _("Product")
    verbose_name_plural = _("Product Gallery")


class ShopProductGallery(WithSortableInlineElementsMixin, ShopPluginBase):
    name = _("Product Gallery")
    require_parent = True
    parent_classes = ('BootstrapColumnPlugin',)
    inlines = (ProductGalleryInline,)
    # until this bug https://github.com/applegrew/django-select2/issues/65 is fixed
    # we hide the a "add row" button and instead use `extra = 5` in ProductGalleryInline

    class Media:
        css = {'all': ('shop/css/admin/product-gallery.css',)}

    def get_render_template(self, context, instance, placeholder):
        return select_template([
            '{}/catalog/product-gallery.html'.format(app_settings.APP_LABEL),
            'shop/catalog/product-gallery.html',
        ])

    def render(self, context, instance, placeholder):
        product_ids = []
        for inline in instance.sortinline_elements.all():
            try:
                product_ids.append(inline.glossary['product']['pk'])
            except KeyError:
                pass
        queryset = ProductModel.objects.filter(pk__in=product_ids)
        serializer_class = app_settings.PRODUCT_SUMMARY_SERIALIZER
        serialized = serializer_class(queryset, many=True, context={'request': context['request']})
        context['products'] = serialized.data
        return context

plugin_pool.register_plugin(ShopProductGallery)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib import admin
from django.utils.html import format_html_join
from django.utils.translation import ugettext_lazy as _

from shop.admin.customer import CustomerProxy, CustomerInlineAdminBase, CustomerAdminBase


class CustomerInlineAdmin(CustomerInlineAdminBase):
    fieldsets = (
        (None, {'fields': ('get_number', 'salutation')}),
        (_("Addresses"), {'fields': ('get_shipping_addresses', 'get_billing_addresses')})
    )
    readonly_fields = ('get_number', 'get_shipping_addresses', 'get_billing_addresses')

    def get_number(self, customer):
        return customer.get_number() or '–'
    get_number.short_description = _("Customer Number")

    def get_shipping_addresses(self, customer):
        addresses = [(a.as_text(),) for a in customer.shippingaddress_set.all()]
        return format_html_join('', '<address>{0}</address>', addresses)
    get_shipping_addresses.short_description = _("Shipping")

    def get_billing_addresses(self, customer):
        addresses = [(a.as_text(),) for a in customer.billingaddress_set.all()]
        return format_html_join('', '<address>{0}</address>', addresses)
    get_billing_addresses.short_description = _("Billing")


@admin.register(CustomerProxy)
class CustomerAdmin(CustomerAdminBase):
    inlines = (CustomerInlineAdmin,)

    def get_list_display(self, request):
        list_display = list(super(CustomerAdmin, self).get_list_display(request))
        list_display.insert(1, 'salutation')
        return list_display

    def salutation(self, user):
        if hasattr(user, 'customer'):
            return user.customer.get_salutation_display()
        return ''
    salutation.short_description = _("Salutation")
    salutation.admin_order_field = 'customer__salutation'



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django_filters import filters

from djng.forms import fields


class Filter(filters.Filter):
    field_class = fields.Field


class CharFilter(filters.CharFilter):
    field_class = fields.CharField


class BooleanFilter(filters.BooleanFilter):
    field_class = fields.NullBooleanField


class ChoiceFilter(filters.ChoiceFilter):
    field_class = fields.ChoiceField


class TypedChoiceFilter(filters.TypedChoiceFilter):
    field_class = fields.TypedChoiceField


class UUIDFilter(filters.UUIDFilter):
    field_class = fields.UUIDField


class MultipleChoiceFilter(filters.MultipleChoiceFilter):
    field_class = fields.MultipleChoiceField


class TypedMultipleChoiceFilter(filters.TypedMultipleChoiceFilter):
    field_class = fields.TypedMultipleChoiceField


class DateFilter(filters.DateFilter):
    field_class = fields.DateField


class DateTimeFilter(filters.DateTimeFilter):
    field_class = fields.DateTimeField


class TimeFilter(filters.TimeFilter):
    field_class = fields.TimeField


class DurationFilter(filters.DurationFilter):
    field_class = fields.DurationField


class ModelChoiceFilter(filters.ModelChoiceFilter):
    field_class = fields.ModelChoiceField


class ModelMultipleChoiceFilter(filters.ModelMultipleChoiceFilter):
    field_class = fields.ModelMultipleChoiceField


class NumberFilter(filters.NumberFilter):
    field_class = fields.DecimalField


class NumericRangeFilter(filters.NumericRangeFilter):
    """
    TODO: we first must redeclare the RangeField
    """



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models
from django.utils.translation import ugettext_lazy as _

from shop.models import order


class OrderItem(order.BaseOrderItem):
    """Default materialized model for OrderItem"""
    quantity = models.IntegerField(_("Ordered quantity"))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.core import exceptions
from django.core.cache import cache
from django.template import TemplateDoesNotExist
from django.template.loader import select_template
from django.utils.html import strip_spaces_between_tags
from django.utils.formats import localize
from django.utils.safestring import mark_safe, SafeText
from django.utils.translation import get_language_from_request

from rest_framework import serializers

from shop.conf import app_settings
from shop.models.customer import CustomerModel
from shop.models.product import ProductModel
from shop.models.order import OrderItemModel
from shop.rest.money import MoneyField


class BaseCustomerSerializer(serializers.ModelSerializer):
    number = serializers.CharField(source='get_number')

    class Meta:
        model = CustomerModel
        fields = ['number', 'first_name', 'last_name', 'email']


class ProductSerializer(serializers.ModelSerializer):
    """
    Common serializer for our product model.
    """
    price = serializers.SerializerMethodField()
    availability = serializers.SerializerMethodField()
    product_type = serializers.CharField(read_only=True)
    product_model = serializers.CharField(read_only=True)
    product_url = serializers.URLField(source='get_absolute_url', read_only=True)

    class Meta:
        model = ProductModel
        fields = '__all__'

    def __init__(self, *args, **kwargs):
        kwargs.setdefault('label', 'catalog')
        super(ProductSerializer, self).__init__(*args, **kwargs)

    def get_price(self, product):
        price = product.get_price(self.context['request'])
        return localize(price)

    def get_availability(self, product):
        return product.get_availability(self.context['request'])

    def render_html(self, product, postfix):
        """
        Return a HTML snippet containing a rendered summary for this product.
        Build a template search path with `postfix` distinction.
        """
        if not self.label:
            msg = "The Product Serializer must be configured using a `label` field."
            raise exceptions.ImproperlyConfigured(msg)
        app_label = product._meta.app_label.lower()
        request = self.context['request']
        cache_key = 'product:{0}|{1}-{2}-{3}-{4}-{5}'.format(product.id, app_label, self.label,
            product.product_model, postfix, get_language_from_request(request))
        content = cache.get(cache_key)
        if content:
            return mark_safe(content)
        params = [
            (app_label, self.label, product.product_model, postfix),
            (app_label, self.label, 'product', postfix),
            ('shop', self.label, 'product', postfix),
        ]
        try:
            template = select_template(['{0}/products/{1}-{2}-{3}.html'.format(*p) for p in params])
        except TemplateDoesNotExist:
            return SafeText("<!-- no such template: '{0}/products/{1}-{2}-{3}.html' -->".format(*params[0]))
        # when rendering emails, we require an absolute URI, so that media can be accessed from
        # the mail client
        absolute_base_uri = request.build_absolute_uri('/').rstrip('/')
        context = {'product': product, 'ABSOLUTE_BASE_URI': absolute_base_uri}
        content = strip_spaces_between_tags(template.render(context, request).strip())
        cache.set(cache_key, content, app_settings.CACHE_DURATIONS['product_html_snippet'])
        return mark_safe(content)


class BaseOrderItemSerializer(serializers.ModelSerializer):
    line_total = MoneyField()
    unit_price = MoneyField()
    product_code = serializers.CharField()

    class Meta:
        model = OrderItemModel



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.views.decorators.cache import never_cache

from rest_framework import generics, mixins
from rest_framework.exceptions import NotFound, PermissionDenied, MethodNotAllowed
from rest_framework.renderers import BrowsableAPIRenderer

from shop.rest.money import JSONRenderer
from shop.rest.renderers import CMSPageRenderer
from shop.serializers.order import OrderListSerializer, OrderDetailSerializer
from shop.models.order import OrderModel


class OrderView(mixins.ListModelMixin, mixins.RetrieveModelMixin, mixins.UpdateModelMixin,
                generics.GenericAPIView):
    """
    Base View class to render the fulfilled orders for the current user.
    """
    renderer_classes = (CMSPageRenderer, JSONRenderer, BrowsableAPIRenderer)
    list_serializer_class = OrderListSerializer
    detail_serializer_class = OrderDetailSerializer
    lookup_field = lookup_url_kwarg = 'slug'
    many = True
    is_last = False

    def get_queryset(self):
        return OrderModel.objects.filter_from_request(self.request)

    def get_serializer_class(self):
        if self.many:
            return self.list_serializer_class
        return self.detail_serializer_class

    def get_renderer_context(self):
        renderer_context = super(OrderView, self).get_renderer_context()
        if self.request.accepted_renderer.format == 'html':
            renderer_context.update(many=self.many, is_last=self.is_last)
            if self.many is False and self.is_last is False:
                # add an extra ance to the breadcrumb to show the order number
                try:
                    renderer_context['extra_ance'] = self.get_object().get_number()
                except (AttributeError, PermissionDenied):
                    pass
        return renderer_context

    def get_object(self):
        if self.lookup_url_kwarg not in self.kwargs:
            return self.get_queryset().first()
        return super(OrderView, self).get_object()

    @property
    def allowed_methods(self):
        """Restrict method "POST" only on the detail view"""
        allowed_methods = self._allowed_methods()
        if self.many:
            allowed_methods.remove('POST')
        return allowed_methods

    @never_cache
    def get(self, request, *args, **kwargs):
        if self.many:
            return self.list(request, *args, **kwargs)
        return self.retrieve(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        if self.many:
            raise MethodNotAllowed("Method POST is not allowed on Order List View")
        self.update(request, *args, **kwargs)
        return self.retrieve(request, *args, **kwargs)

    def list(self, request, *args, **kwargs):
        try:
            return super(OrderView, self).list(request, *args, **kwargs)
        except OrderModel.DoesNotExist:
            raise NotFound("No orders have been found for the current user.")

    def retrieve(self, request, *args, **kwargs):
        try:
            return super(OrderView, self).retrieve(request, *args, **kwargs)
        except OrderModel.DoesNotExist:
            raise NotFound("No order has been found for the current user.")



# -*- coding: utf-8 -*-
# Generated by Django 1.10.4 on 2016-12-27 09:08
from __future__ import unicode_literals

from django import VERSION
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('email_auth', '0002_auto_20160327_1119'),
    ]

    operations = []
    if VERSION >= (1, 10):
        import django.contrib.auth.validators

        operations.append(migrations.AlterField(
            model_name='user',
            name='username',
            field=models.CharField(error_messages={'unique': 'A user with that username already exists.'}, help_text='Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.', max_length=150, unique=True, validators=[django.contrib.auth.validators.ASCIIUsernameValidator()], verbose_name='username'),
        ))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
"""
Holds all the information relevant to the client (addresses for instance)
"""
from six import with_metaclass

from django.db import models
from django.template import Context
from django.template.loader import select_template
from django.utils.translation import ugettext_lazy as _

from shop import deferred
from shop.conf import app_settings


class AddressManager(models.Manager):
    def get_max_priority(self, customer):
        aggr = self.get_queryset().filter(customer=customer).aggregate(models.Max('priority'))
        priority = aggr['priority__max'] or 0
        return priority

    def get_fallback(self, customer):
        """
        Return a fallback address, whenever the customer has not declared one.
        """
        return self.get_queryset().filter(customer=customer).order_by('priority').last()


class BaseAddress(models.Model):
    customer = deferred.ForeignKey('BaseCustomer')

    priority = models.SmallIntegerField(
        default=0,
        db_index=True,
        help_text=_("Priority for using this address"),
    )

    class Meta:
        abstract = True

    objects = AddressManager()

    def as_text(self):
        """
        Return the address as plain text to be used for printing, etc.
        """
        template_names = [
            '{}/{}-address.txt'.format(app_settings.APP_LABEL, self.address_type),
            '{}/address.txt'.format(app_settings.APP_LABEL),
            'shop/address.txt',
        ]
        template = select_template(template_names)
        context = Context({'address': self})
        return template.render(context)
    as_text.short_description = _("Address")


class BaseShippingAddress(with_metaclass(deferred.ForeignKeyBuilder, BaseAddress)):
    address_type = 'shipping'

    class Meta:
        abstract = True

ShippingAddressModel = deferred.MaterializedModel(BaseShippingAddress)


class BaseBillingAddress(with_metaclass(deferred.ForeignKeyBuilder, BaseAddress)):
    address_type = 'billing'

    class Meta:
        abstract = True

BillingAddressModel = deferred.MaterializedModel(BaseBillingAddress)

ISO_3166_CODES = (
    ('AF', _("Afghanistan")),
    ('AX', _("Aland Islands")),
    ('AL', _("Albania")),
    ('DZ', _("Algeria")),
    ('AS', _("American Samoa")),
    ('AD', _("Andorra")),
    ('AO', _("Angola")),
    ('AI', _("Anguilla")),
    ('AQ', _("Antarctica")),
    ('AG', _("Antigua And Barbuda")),
    ('AR', _("Argentina")),
    ('AM', _("Armenia")),
    ('AW', _("Aruba")),
    ('AU', _("Australia")),
    ('AT', _("Austria")),
    ('AZ', _("Azerbaijan")),
    ('BS', _("Bahamas")),
    ('BH', _("Bahrain")),
    ('BD', _("Bangladesh")),
    ('BB', _("Barbados")),
    ('BY', _("Belarus")),
    ('BE', _("Belgium")),
    ('BZ', _("Belize")),
    ('BJ', _("Benin")),
    ('BM', _("Bermuda")),
    ('BT', _("Bhutan")),
    ('BO', _("Bolivia, Plurinational State Of")),
    ('BQ', _("Bonaire, Saint Eustatius And Saba")),
    ('BA', _("Bosnia And Herzegovina")),
    ('BW', _("Botswana")),
    ('BV', _("Bouvet Island")),
    ('BR', _("Brazil")),
    ('IO', _("British Indian Ocean Territory")),
    ('BN', _("Brunei Darussalam")),
    ('BG', _("Bulgaria")),
    ('BF', _("Burkina Faso")),
    ('BI', _("Burundi")),
    ('KH', _("Cambodia")),
    ('CM', _("Cameroon")),
    ('CA', _("Canada")),
    ('CV', _("Cape Verde")),
    ('KY', _("Cayman Islands")),
    ('CF', _("Central African Republic")),
    ('TD', _("Chad")),
    ('CL', _("Chile")),
    ('CN', _("China")),
    ('CX', _("Christmas Island")),
    ('CC', _("Cocos (Keeling) Islands")),
    ('CO', _("Colombia")),
    ('KM', _("Comoros")),
    ('CG', _("Congo")),
    ('CD', _("Congo, The Democratic Republic Of The")),
    ('CK', _("Cook Islands")),
    ('CR', _("Costa Rica")),
    ('HR', _("Croatia")),
    ('CU', _("Cuba")),
    ('CW', _("Curacao")),
    ('CY', _("Cyprus")),
    ('CZ', _("Czech Republic")),
    ('DK', _("Denmark")),
    ('DJ', _("Djibouti")),
    ('DM', _("Dominica")),
    ('DO', _("Dominican Republic")),
    ('EC', _("Ecuador")),
    ('EG', _("Egypt")),
    ('SV', _("El Salvador")),
    ('GQ', _("Equatorial Guinea")),
    ('ER', _("Eritrea")),
    ('EE', _("Estonia")),
    ('ET', _("Ethiopia")),
    ('FK', _("Falkland Islands (Malvinas)")),
    ('FO', _("Faroe Islands")),
    ('FJ', _("Fiji")),
    ('FI', _("Finland")),
    ('FR', _("France")),
    ('GF', _("French Guiana")),
    ('PF', _("French Polynesia")),
    ('TF', _("French Southern Territories")),
    ('GA', _("Gabon")),
    ('GM', _("Gambia")),
    ('DE', _("Germany")),
    ('GH', _("Ghana")),
    ('GI', _("Gibraltar")),
    ('GR', _("Greece")),
    ('GL', _("Greenland")),
    ('GD', _("Grenada")),
    ('GP', _("Guadeloupe")),
    ('GU', _("Guam")),
    ('GT', _("Guatemala")),
    ('GG', _("Guernsey")),
    ('GN', _("Guinea")),
    ('GW', _("Guinea-Bissau")),
    ('GY', _("Guyana")),
    ('HT', _("Haiti")),
    ('HM', _("Heard Island and McDonald Islands")),
    ('VA', _("Holy See (Vatican City State)")),
    ('HN', _("Honduras")),
    ('HK', _("Hong Kong")),
    ('HU', _("Hungary")),
    ('IS', _("Iceland")),
    ('IN', _("India")),
    ('ID', _("Indonesia")),
    ('IR', _("Iran, Islamic Republic Of")),
    ('IQ', _("Iraq")),
    ('IE', _("Ireland")),
    ('IL', _("Israel")),
    ('IT', _("Italy")),
    ('CI', _("Ivory Coast")),
    ('JM', _("Jamaica")),
    ('JP', _("Japan")),
    ('JE', _("Jersey")),
    ('JO', _("Jordan")),
    ('KZ', _("Kazakhstan")),
    ('KE', _("Kenya")),
    ('KI', _("Kiribati")),
    ('KP', _("Korea, Democratic People's Republic Of")),
    ('KR', _("Korea, Republic Of")),
    ('KS', _("Kosovo")),
    ('KW', _("Kuwait")),
    ('KG', _("Kyrgyzstan")),
    ('LA', _("Lao People's Democratic Republic")),
    ('LV', _("Latvia")),
    ('LB', _("Lebanon")),
    ('LS', _("Lesotho")),
    ('LR', _("Liberia")),
    ('LY', _("Libyan Arab Jamahiriya")),
    ('LI', _("Liechtenstein")),
    ('LT', _("Lithuania")),
    ('LU', _("Luxembourg")),
    ('MO', _("Macao")),
    ('MK', _("Macedonia")),
    ('MG', _("Madagascar")),
    ('MW', _("Malawi")),
    ('MY', _("Malaysia")),
    ('MV', _("Maldives")),
    ('ML', _("Mali")),
    ('ML', _("Malta")),
    ('MH', _("Marshall Islands")),
    ('MQ', _("Martinique")),
    ('MR', _("Mauritania")),
    ('MU', _("Mauritius")),
    ('YT', _("Mayotte")),
    ('MX', _("Mexico")),
    ('FM', _("Micronesia")),
    ('MD', _("Moldova")),
    ('MC', _("Monaco")),
    ('MN', _("Mongolia")),
    ('ME', _("Montenegro")),
    ('MS', _("Montserrat")),
    ('MA', _("Morocco")),
    ('MZ', _("Mozambique")),
    ('MM', _("Myanmar")),
    ('NA', _("Namibia")),
    ('NR', _("Nauru")),
    ('NP', _("Nepal")),
    ('NL', _("Netherlands")),
    ('AN', _("Netherlands Antilles")),
    ('NC', _("New Caledonia")),
    ('NZ', _("New Zealand")),
    ('NI', _("Nicaragua")),
    ('NE', _("Niger")),
    ('NG', _("Nigeria")),
    ('NU', _("Niue")),
    ('NF', _("Norfolk Island")),
    ('MP', _("Northern Mariana Islands")),
    ('NO', _("Norway")),
    ('OM', _("Oman")),
    ('PK', _("Pakistan")),
    ('PW', _("Palau")),
    ('PS', _("Palestinian Territory, Occupied")),
    ('PA', _("Panama")),
    ('PG', _("Papua New Guinea")),
    ('PY', _("Paraguay")),
    ('PE', _("Peru")),
    ('PH', _("Philippines")),
    ('PN', _("Pitcairn")),
    ('PL', _("Poland")),
    ('PT', _("Portugal")),
    ('PR', _("Puerto Rico")),
    ('QA', _("Qatar")),
    ('RE', _("Reunion")),
    ('RO', _("Romania")),
    ('RU', _("Russian Federation")),
    ('RW', _("Rwanda")),
    ('BL', _("Saint Barthelemy")),
    ('SH', _("Saint Helena, Ascension & Tristan Da Cunha")),
    ('KN', _("Saint Kitts and Nevis")),
    ('LC', _("Saint Lucia")),
    ('MF', _("Saint Martin (French Part)")),
    ('PM', _("Saint Pierre and Miquelon")),
    ('VC', _("Saint Vincent And The Grenadines")),
    ('WS', _("Samoa")),
    ('SM', _("San Marino")),
    ('ST', _("Sao Tome And Principe")),
    ('SA', _("Saudi Arabia")),
    ('SN', _("Senegal")),
    ('RS', _("Serbia")),
    ('SC', _("Seychelles")),
    ('SL', _("Sierra Leone")),
    ('SG', _("Singapore")),
    ('SX', _("Sint Maarten (Dutch Part)")),
    ('SK', _("Slovakia")),
    ('SI', _("Slovenia")),
    ('SB', _("Solomon Islands")),
    ('SO', _("Somalia")),
    ('ZA', _("South Africa")),
    ('GS', _("South Georgia And The South Sandwich Islands")),
    ('ES', _("Spain")),
    ('LK', _("Sri Lanka")),
    ('SD', _("Sudan")),
    ('SR', _("Suriname")),
    ('SJ', _("Svalbard And Jan Mayen")),
    ('SZ', _("Swaziland")),
    ('SE', _("Sweden")),
    ('CH', _("Switzerland")),
    ('SY', _("Syrian Arab Republic")),
    ('TW', _("Taiwan")),
    ('TJ', _("Tajikistan")),
    ('TZ', _("Tanzania")),
    ('TH', _("Thailand")),
    ('TL', _("Timor-Leste")),
    ('TG', _("Togo")),
    ('TK', _("Tokelau")),
    ('TO', _("Tonga")),
    ('TT', _("Trinidad and Tobago")),
    ('TN', _("Tunisia")),
    ('TR', _("Turkey")),
    ('TM', _("Turkmenistan")),
    ('TC', _("Turks And Caicos Islands")),
    ('TV', _("Tuvalu")),
    ('UG', _("Uganda")),
    ('UA', _("Ukraine")),
    ('AE', _("United Arab Emirates")),
    ('GB', _("United Kingdom")),
    ('US', _("United States")),
    ('UM', _("United States Minor Outlying Islands")),
    ('UY', _("Uruguay")),
    ('UZ', _("Uzbekistan")),
    ('VU', _("Vanuatu")),
    ('VE', _("Venezuela, Bolivarian Republic Of")),
    ('VN', _("Viet Nam")),
    ('VG', _("Virgin Islands, British")),
    ('VI', _("Virgin Islands, U.S.")),
    ('WF', _("Wallis and Futuna")),
    ('EH', _("Western Sahara")),
    ('YE', _("Yemen")),
    ('ZM', _("Zambia")),
    ('ZW', _("Zimbabwe")),
)

class CountryField(models.CharField):
    """
    This creates a simple input field to choose a country.
    """
    def __init__(self, *args, **kwargs):
        defaults = {
            'max_length': 3,
            'choices': ISO_3166_CODES,
        }
        defaults.update(kwargs)
        super(CountryField, self).__init__(*args, **defaults)

    def deconstruct(self):
        name, path, args, kwargs = super(CountryField, self).deconstruct()
        if kwargs['max_length'] == 3:
            kwargs.pop('max_length')
        if kwargs['choices'] == ISO_3166_CODES:
            kwargs.pop('choices')
        return name, path, args, kwargs



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from adminsortable2.admin import SortableAdminMixin
from cms.admin.placeholderadmin import PlaceholderAdminMixin, FrontendEditableAdminMixin
from shop.admin.product import CMSPageAsCategoryMixin, CMSPageFilter
from shop.models.defaults.commodity import Commodity

if settings.USE_I18N:
    from parler.admin import TranslatableAdmin


    @admin.register(Commodity)
    class CommodityAdmin(SortableAdminMixin, TranslatableAdmin, FrontendEditableAdminMixin,
                         PlaceholderAdminMixin, CMSPageAsCategoryMixin, admin.ModelAdmin):
        fieldsets = (
            (None, {
                'fields': ('product_name', 'slug', 'caption',)
            }),
            (_("Common Fields"), {
                'fields': ('product_code', ('unit_price', 'active',), 'show_breadcrumb', 'sample_image'),
            }),
        )
        filter_horizontal = ('cms_pages',)
        list_filter = (CMSPageFilter,)

        def get_prepopulated_fields(self, request, obj=None):
            return {
                'slug': ('product_name',)
            }

else:

    @admin.register(Commodity)
    class CommodityAdmin(SortableAdminMixin, FrontendEditableAdminMixin, PlaceholderAdminMixin,
                         CMSPageAsCategoryMixin, admin.ModelAdmin):
        fields = ('product_name', 'slug',  'caption', 'product_code',
                  ('unit_price', 'active',), 'show_breadcrumb', 'sample_image',)
        filter_horizontal = ('cms_pages',)
        prepopulated_fields = {'slug': ('product_name',)}



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from functools import reduce
import operator
from django.core.exceptions import ImproperlyConfigured
from django.db.models import Q
from rest_framework.filters import BaseFilterBackend


class CMSPagesFilterBackend(BaseFilterBackend):
    """
    Use this backend to only show products assigned to the current page.
    """

    cms_pages_fields = ['cms_pages']

    def _get_filtered_queryset(self, current_page, queryset, cms_pages_fields):
        filter_by_cms_page = (Q((field, current_page)) for field in cms_pages_fields)
        return queryset.filter(reduce(operator.or_, filter_by_cms_page)).distinct()

    def filter_queryset(self, request, queryset, view):
        cms_pages_fields = getattr(view, 'cms_pages_fields', self.cms_pages_fields)
        if not isinstance(cms_pages_fields, (list, tuple)):
            msg = "`cms_pages_fields` must be a list or tuple of fields referring to djangoCMS pages."
            raise ImproperlyConfigured(msg)
        current_page = request.current_page
        if current_page.publisher_is_draft:
            current_page = current_page.publisher_public
        return self._get_filtered_queryset(current_page, queryset, cms_pages_fields)


class RecursiveCMSPagesFilterBackend(CMSPagesFilterBackend):
    """
    Use this backend to show products assigned to the current page or any of its descendants.
    """

    def _get_filtered_queryset(self, current_page, queryset, cms_pages_fields):
        pages = current_page.get_descendants(include_self=True)
        filter_by_cms_page = (Q((field + "__in", pages)) for field in self.cms_pages_fields)
        return queryset.filter(reduce(operator.or_, filter_by_cms_page)).distinct()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.conf import settings
from django.db import models, migrations
import filer.fields.file


class Migration(migrations.Migration):

    dependencies = [
        ('filer', '0002_auto_20150606_2003'),
        ('post_office', '0002_add_i18n_and_backend_alias'),
    ]
    if 'email_auth' in settings.INSTALLED_APPS:
        dependencies.append(('email_auth', '0001_initial'))
        customer_bases = ('email_auth.user',)
    else:
        dependencies.append(('auth', '0001_initial'))
        customer_bases = ('auth.user',)

    operations = [
        migrations.CreateModel(
            name='Notification',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=50, verbose_name='Name')),
                ('transition_target', models.CharField(max_length=50, verbose_name='Event')),
                ('mail_to', models.PositiveIntegerField(default=None, null=True, verbose_name='Mail to', blank=True)),
                ('mail_template', models.ForeignKey(verbose_name='Template', to='post_office.EmailTemplate')),
            ],
            options={
                'verbose_name': 'Notification',
                'verbose_name_plural': 'Notifications',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='NotificationAttachment',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('attachment', filer.fields.file.FilerFileField(related_name='email_attachment', blank=True, to='filer.File', null=True)),
                ('notification', models.ForeignKey(to='shop.Notification')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='CustomerProxy',
            fields=[
            ],
            options={
                'verbose_name': 'Customer',
                'proxy': True,
                'verbose_name_plural': 'Customers',
            },
            bases=customer_bases,
        ),
        migrations.CreateModel(
            name='Email',
            fields=[
            ],
            options={
                'proxy': True,
            },
            bases=('post_office.email',),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
"""
See PEP 386 (http://www.python.org/dev/peps/pep-0386/)

Release logic:
 1. Remove ".devX" from __version__ (below)
 2. git add shop/__init__.py
 3. git commit -m 'Bump to <version>'
 4. git tag <version>
 5. git push
 6. assure that all tests pass on https://travis-ci.org/awesto/django-shop
 7. git push --tags
 8. python setup.py sdist upload
 9. bump the version, append ".dev0" to __version__
10. git add shop/__init__.py
11. git commit -m 'Start with <version>'
12. git push
"""
__version__ = '0.11.2'

default_app_config = 'shop.apps.ShopConfig'



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-04-11 15:33
from __future__ import unicode_literals

from django import VERSION

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('email_auth', '0003_django110'),
    ]

    operations = []

    if VERSION >= (1, 10):
        import django.contrib.auth.validators

        operations.append(
            migrations.AlterField(
                model_name='user',
                name='username',
                field=models.CharField(error_messages={'unique': 'A user with that username already exists.'}, help_text='Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.', max_length=150, unique=True, validators=[django.contrib.auth.validators.ASCIIUsernameValidator()], verbose_name='username'),
            )
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db.models import Sum
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _
from django_fsm import transition
from shop.models.delivery import DeliveryModel


class PartialDeliveryWorkflowMixin(object):
    """
    Add this class to `settings.SHOP_ORDER_WORKFLOWS` to mix it into the `OrderModel`.
    This mixin supports partial delivery, hence the model `Delivery` and `DeliveryItem` must
    be materialized.
    It adds all the methods required for state transitions, while picking, packing and shipping
    the ordered goods for delivery.
    """
    TRANSITION_TARGETS = {
        'pick_goods': _("Picking goods"),
        'pack_goods': _("Packing goods"),
        'ship_goods': _("Ship goods"),
    }

    @cached_property
    def unfulfilled_items(self):
        unfulfilled_items = 0
        for order_item in self.items.all():
            if not order_item.canceled:
                aggr = order_item.deliveryitem_set.aggregate(delivered=Sum('quantity'))
                unfulfilled_items += order_item.quantity - (aggr['delivered'] or 0)
        return unfulfilled_items

    def ready_for_delivery(self):
        return self.is_fully_paid() and self.unfulfilled_items > 0

    @transition(field='status', source=['payment_confirmed', 'pack_goods', 'ship_goods'],
                target='pick_goods', conditions=[ready_for_delivery],
        custom=dict(admin=True, button_name=_("Pick the goods")))
    def pick_goods(self, by=None):
        """Change status to 'pick_goods'."""
        shipping_method = self.extra.get('shipping_modifier')
        if shipping_method:
            DeliveryModel.objects.get_or_create(order=self, shipping_method=shipping_method, fulfilled_at=None)

    @transition(field='status', source=['pick_goods'], target='pack_goods',
        custom=dict(admin=True, button_name=_("Pack the goods")))
    def pack_goods(self, by=None):
        """Prepare shipping object and change status to 'pack_goods'."""
        self._transition_to_pack_goods = True  # hack to determine this transition in admin backend

    @transition(field='status', source=['pack_goods'], target='ship_goods',
        custom=dict(admin=True, button_name=_("Ship the goods")))
    def ship_goods(self, by=None):
        """Use selected shipping object and change status to 'ship_goods'."""



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
"""
Some models in the merchant's implementation require a many-to-many relation with models from
outside django-SHOP. Therefore these mapping tables must be materialized by the merchant's
implementation.
"""
from shop.models.related import BaseProductPage, BaseProductImage


class ProductPage(BaseProductPage):
    """Materialize many-to-many relation with CMS pages"""


class ProductImage(BaseProductImage):
    """Materialize many-to-many relation with images"""
    class Meta(BaseProductImage.Meta):
        abstract = False



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import email_auth.models
import re
import django.utils.timezone
import django.core.validators


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(default=django.utils.timezone.now, verbose_name='last login')),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status')),
                ('username', models.CharField(help_text='Required. 30 characters or fewer. Letters, numbers and @/./+/-/_ characters', unique=True, max_length=30, verbose_name='Username', validators=[django.core.validators.RegexValidator(re.compile('^[\\w.@+-]+$'), 'Enter a valid username.', 'invalid')])),
                ('first_name', models.CharField(max_length=30, verbose_name='First name', blank=True)),
                ('last_name', models.CharField(max_length=30, verbose_name='Last name', blank=True)),
                ('email', models.EmailField(default=None, max_length=254, null=True, verbose_name='Email address', blank=True)),
                ('is_staff', models.BooleanField(default=False, help_text='Designates whether the user can log into this admin site.', verbose_name='staff status')),
                ('is_active', models.BooleanField(default=True, help_text='Designates whether this user should be treated as active.Unselect this instead of deleting accounts.', verbose_name='active')),
                ('date_joined', models.DateTimeField(default=django.utils.timezone.now, verbose_name='date joined')),
                ('groups', models.ManyToManyField(related_query_name='user', related_name='user_set', to='auth.Group', blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of his/her group.', verbose_name='groups')),
                ('user_permissions', models.ManyToManyField(related_query_name='user', related_name='user_set', to='auth.Permission', blank=True, help_text='Specific permissions for this user.', verbose_name='user permissions')),
            ],
            options={
                'db_table': 'auth_user',
                'verbose_name': 'Customer',
                'verbose_name_plural': 'Customers',
            },
        ),
        migrations.AlterModelManagers(
            name='user',
            managers=[
                ('objects', email_auth.models.UserManager()),
            ],
        ),
        migrations.AlterField(
            model_name='user',
            name='groups',
            field=models.ManyToManyField(related_query_name='user', related_name='user_set', to='auth.Group', blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', verbose_name='groups'),
        ),
        migrations.AlterField(
            model_name='user',
            name='last_login',
            field=models.DateTimeField(null=True, verbose_name='last login', blank=True),
        ),
        migrations.AlterField(
            model_name='user',
            name='email',
            field=models.EmailField(default=None, max_length=254, verbose_name='email address', blank=True),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='user',
            name='first_name',
            field=models.CharField(max_length=30, verbose_name='first name', blank=True),
        ),
        migrations.AlterField(
            model_name='user',
            name='is_active',
            field=models.BooleanField(default=True, help_text='Designates whether this user should be treated as active. Unselect this instead of deleting accounts.', verbose_name='active'),
        ),
        migrations.AlterField(
            model_name='user',
            name='last_name',
            field=models.CharField(max_length=30, verbose_name='last name', blank=True),
        ),
        migrations.AlterField(
            model_name='user',
            name='username',
            field=models.CharField(error_messages={'unique': 'A user with that username already exists.'}, max_length=30, validators=[django.core.validators.RegexValidator('^[\\w.@+-]+$', 'Enter a valid username. This value may contain only letters, numbers and @/./+/-/_ characters.', 'invalid')], help_text='Required. 30 characters or fewer. Letters, digits and @/./+/-/_ only.', unique=True, verbose_name='username'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.utils.translation import ugettext_lazy as _

# Dictionary of currency representations:
# key: official ISO 4217 code
# value[0]: numeric representation
# value[1]: number of digits
# value[2]: currency symbol in UTF-8
# value[3]: textual description
CURRENCIES = {
    'AED': ('784', 2, 'د.إ', _('United Arab Emirates dirham')),
    'AUD': ('036', 2, '$', _("Australian Dollar")),
    'BHD': ('048', 3, '.د.ب', _('Bahraini dinar')),
    'BRL': ('986', 2, 'R$', _("Brazilian Real")),
    'CAD': ('124', 2, 'C$', _("Canadian Dollar")),
    'CHF': ('756', 2, 'SFr․', _("Swiss Franc")),  # Unicode 8228 as dot to prevent formatting issues
    'CNY': ('156', 2, '¥', _("Chinese Yuan")),
    'CZK': ('203', 2, 'Kč', _("Czech Koruna")),
    'EUR': ('978', 2, '€', _("Euro")),
    'GBP': ('826', 2, '£', _("Pound Sterling")),
    'HRK': ('191', 2, 'kn', _("Croatian kuna")),
    'HUF': ('348', 0, 'Ft', _("Hungarian Forint")),
    'ILS': ('376', 2, '₪', _("Israeli Sheqel")),
    'JPY': ('392', 0, '¥', _("Japanese Yen")),
    'KWD': ('414', 3, 'د.ك', _("Kuwaiti Dinar")),
    'OMR': ('512', 3, 'ر.ع.', _('Omani rial')),
    'QAR': ('634', 2, 'ر.ق', _('Qatari riyal')),
    'RUB': ('643', 2, '₽', _("Russian Ruble")),
    'SAR': ('682', 2, 'ر.س', _('Saudi riyal')),
    'TND': ('788', 3, 'TND', _("Tunisian Dinar")),
    'UAH': ('980', 2, '₴', _("Ukrainian Hryvnia")),
    'USD': ('840', 2, '$', _("US Dollar")),
    # feel free to add more currencies, alphabetically ordered
}



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from six import with_metaclass
from collections import OrderedDict
from django.db import models
from django.core.exceptions import ImproperlyConfigured
from django.utils.translation import ugettext_lazy as _
from shop.models.fields import JSONField
from shop.modifiers.pool import cart_modifiers_pool
from shop.money import Money
from .product import BaseProduct
from shop import deferred
from shop.models.customer import CustomerModel


class CartItemManager(models.Manager):
    """
    Customized model manager for our CartItem model.
    """

    def get_or_create(self, **kwargs):
        """
        Create a unique cart item. If the same product exists already in the given cart,
        increase its quantity, if the product in the cart seems to be the same.
        """
        cart = kwargs.pop('cart')
        product = kwargs.pop('product')
        quantity = int(kwargs.pop('quantity', 1))

        # add a new item to the cart, or reuse an existing one, increasing the quantity
        watched = not quantity
        cart_item = product.is_in_cart(cart, watched=watched, **kwargs)
        if cart_item:
            if not watched:
                cart_item.quantity += quantity
            created = False
        else:
            cart_item = self.model(cart=cart, product=product, quantity=quantity, **kwargs)
            created = True

        cart_item.save()
        return cart_item, created

    def filter_cart_items(self, cart, request):
        """
        Use this method to fetch items for shopping from the cart. It rearranges the result set
        according to the defined modifiers.
        """
        cart_items = self.filter(cart=cart, quantity__gt=0).order_by('id')
        for modifier in cart_modifiers_pool.get_all_modifiers():
            cart_items = modifier.arrange_cart_items(cart_items, request)
        return cart_items

    def filter_watch_items(self, cart, request):
        """
        Use this method to fetch items from the watch list. It rearranges the result set
        according to the defined modifiers.
        """
        watch_items = self.filter(cart=cart, quantity=0)
        for modifier in cart_modifiers_pool.get_all_modifiers():
            watch_items = modifier.arrange_watch_items(watch_items, request)
        return watch_items


class BaseCartItem(with_metaclass(deferred.ForeignKeyBuilder, models.Model)):
    """
    This is a holder for the quantity of items in the cart and, obviously, a
    pointer to the actual Product being purchased
    """
    cart = deferred.ForeignKey(
        'BaseCart',
        related_name='items',
    )

    product = deferred.ForeignKey(BaseProduct)

    product_code = models.CharField(
        _("Product code"),
        max_length=255,
        null=True,
        blank=True,
        help_text=_("Product code of added item."),
    )

    extra = JSONField(verbose_name=_("Arbitrary information for this cart item"))

    objects = CartItemManager()

    class Meta:
        abstract = True
        verbose_name = _("Cart item")
        verbose_name_plural = _("Cart items")

    @classmethod
    def perform_model_checks(cls):
        try:
            allowed_types = ('IntegerField', 'DecimalField', 'FloatField')
            field = [f for f in cls._meta.fields if f.attname == 'quantity'][0]
            if not field.get_internal_type() in allowed_types:
                msg = "Field `{}.quantity` must be of one of the types: {}."
                raise ImproperlyConfigured(msg.format(cls.__name__, allowed_types))
        except IndexError:
            msg = "Class `{}` must implement a field named `quantity`."
            raise ImproperlyConfigured(msg.format(cls.__name__))

    def __init__(self, *args, **kwargs):
        # reduce the given fields to what the model actually can consume
        all_field_names = [field.name for field in self._meta.get_fields(include_parents=True)]
        model_kwargs = {k: v for k, v in kwargs.items() if k in all_field_names}
        super(BaseCartItem, self).__init__(*args, **model_kwargs)
        self.extra_rows = OrderedDict()
        self._dirty = True

    def save(self, *args, **kwargs):
        super(BaseCartItem, self).save(*args, **kwargs)
        self._dirty = True
        self.cart._dirty = True

    def update(self, request):
        """
        Loop over all registered cart modifier, change the price per cart item and optionally add
        some extra rows.
        """
        if not self._dirty:
            return
        self.extra_rows = OrderedDict()  # reset the dictionary
        for modifier in cart_modifiers_pool.get_all_modifiers():
            modifier.process_cart_item(self, request)
        self._dirty = False

CartItemModel = deferred.MaterializedModel(BaseCartItem)


class CartManager(models.Manager):
    """
    The Model Manager for any Cart inheriting from BaseCart.
    """

    def get_from_request(self, request):
        """
        Return the cart for current customer.
        """
        if request.customer.is_visitor():
            raise self.model.DoesNotExist("Cart for visiting customer does not exist.")
        cart, created = self.get_or_create(customer=request.customer)
        return cart

    def get_or_create_from_request(self, request):
        if request.customer.is_visitor():
            request.customer = CustomerModel.objects.get_or_create_from_request(request)
        cart, created = self.get_or_create(customer=request.customer)
        return cart


class BaseCart(with_metaclass(deferred.ForeignKeyBuilder, models.Model)):
    """
    The fundamental part of a shopping cart.
    """
    customer = deferred.OneToOneField(
        'BaseCustomer',
        verbose_name=_("Customer"),
        related_name='cart',
    )

    created_at = models.DateTimeField(
        _("Created at"),
        auto_now_add=True,
    )

    updated_at = models.DateTimeField(
        _("Updated at"),
        auto_now=True,
    )

    extra = JSONField(verbose_name=_("Arbitrary information for this cart"))

    # our CartManager determines the cart object from the request.
    objects = CartManager()

    class Meta:
        abstract = True
        verbose_name = _("Shopping Cart")
        verbose_name_plural = _("Shopping Carts")

    def __init__(self, *args, **kwargs):
        super(BaseCart, self).__init__(*args, **kwargs)
        # That will hold things like tax totals or total discount
        self.extra_rows = OrderedDict()
        self._cached_cart_items = None
        self._dirty = True

    def save(self, force_update=False, *args, **kwargs):
        if self.pk or force_update is False:
            super(BaseCart, self).save(force_update=force_update, *args, **kwargs)
        self._dirty = True

    def update(self, request):
        """
        This should be called after a cart item changed quantity, has been added or removed.

        It will loop on all line items in the cart, and call all the cart modifiers for each item.
        After doing this, it will compute and update the order's total and subtotal fields, along
        with any supplement added along the way by modifiers.

        Note that theses added fields are not stored - we actually want to
        reflect rebate and tax changes on the *cart* items, but we don't want
        that for the order items (since they are legally binding after the
        "purchase" button was pressed)
        """
        if not self._dirty:
            return

        if self._cached_cart_items:
            items = self._cached_cart_items
        else:
            items = CartItemModel.objects.filter_cart_items(self, request)

        # This calls all the pre_process_cart methods and the pre_process_cart_item for each item,
        # before processing the cart. This allows to prepare and collect data on the cart.
        for modifier in cart_modifiers_pool.get_all_modifiers():
            modifier.pre_process_cart(self, request)
            for item in items:
                modifier.pre_process_cart_item(self, item, request)

        self.extra_rows = OrderedDict()  # reset the dictionary
        self.subtotal = 0  # reset the subtotal
        for item in items:
            # item.update iterates over all cart modifiers and invokes method `process_cart_item`
            item.update(request)
            self.subtotal += item.line_total

        # Iterate over the registered modifiers, to process the cart's summary
        for modifier in cart_modifiers_pool.get_all_modifiers():
            for item in items:
                modifier.post_process_cart_item(self, item, request)
            modifier.process_cart(self, request)

        # This calls the post_process_cart method from cart modifiers, if any.
        # It allows for a last bit of processing on the "finished" cart, before
        # it is displayed
        for modifier in reversed(cart_modifiers_pool.get_all_modifiers()):
            modifier.post_process_cart(self, request)

        # Cache updated cart items
        self._cached_cart_items = items
        self._dirty = False

    def empty(self):
        """
        Remove the cart with all its items.
        """
        if self.pk:
            self.items.all().delete()
            self.delete()

    def merge_with(self, other_cart):
        """
        Merge the contents of the other cart into this one, afterwards delete it.
        This is done item by item, so that duplicate items increase the quantity.
        """
        # iterate over the cart and add quantities for items from other cart considered as equal
        for item in self.items.all():
            other_item = item.product.is_in_cart(other_cart, extra=item.extra)
            if other_item:
                item.quantity += other_item.quantity
                item.save()
                other_item.delete()

        # the remaining items from the other cart are merged into this one
        other_cart.items.update(cart=self)
        other_cart.delete()

    def __str__(self):
        return "{}".format(self.pk) if self.pk else '(unsaved)'

    @property
    def num_items(self):
        """
        Returns the number of items in the cart.
        """
        return self.items.filter(quantity__gt=0).count()

    @property
    def total_quantity(self):
        """
        Returns the total quantity of all items in the cart.
        """
        aggr = self.items.aggregate(quantity=models.Sum('quantity'))
        return aggr['quantity'] or 0
        # if we would know, that self.items is already evaluated, then this might be faster:
        # return sum([ci.quantity for ci in self.items.all()])

    @property
    def is_empty(self):
        return self.total_quantity == 0

    def get_caption_data(self):
        return {'num_items': self.num_items, 'total_quantity': self.total_quantity,
                'subtotal': self.subtotal, 'total': self.total}

    @classmethod
    def get_default_caption_data(cls):
        return {'num_items': 0, 'total_quantity': 0, 'subtotal': Money(), 'total': Money()}

CartModel = deferred.MaterializedModel(BaseCart)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from shop.conf import app_settings
from shop.serializers.cart import ExtraCartRow
from .base import BaseCartModifier


class CartIncludeTaxModifier(BaseCartModifier):
    """
    This tax calculator presumes that unit prices are net prices, hence also the subtotal,
    and that the tax is added globally to the carts total.
    By placing this modifier after the shipping modifiers, one can add tax to
    the shipping costs. Otherwise shipping cost are considered tax free.
    """
    taxes = app_settings.VALUE_ADDED_TAX / 100

    def add_extra_cart_row(self, cart, request):
        """
        Add a field on cart.extra_price_fields:
        """
        amount = cart.subtotal * self.taxes
        instance = {
            'label': _("plus {}% VAT").format(app_settings.VALUE_ADDED_TAX),
            'amount': amount,
        }
        cart.extra_rows[self.identifier] = ExtraCartRow(instance)
        cart.total += amount


class CartExcludedTaxModifier(BaseCartModifier):
    """
    This tax calculator presumes that unit prices are gross prices, hence also the subtotal,
    and that the tax is calculated per cart but not added to the cart.
    """
    taxes = 1 - 1 / (1 + app_settings.VALUE_ADDED_TAX / 100)

    def add_extra_cart_row(self, cart, request):
        """
        Add a field on cart.extra_price_fields:
        """
        amount = cart.subtotal * self.taxes
        instance = {
            'label': _("{}% VAT incl.").format(app_settings.VALUE_ADDED_TAX),
            'amount': amount,
        }
        cart.extra_rows[self.identifier] = ExtraCartRow(instance)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.core.exceptions import ImproperlyConfigured
from django.utils.translation import ugettext_lazy as _

from cms.app_base import CMSApp


class CatalogListCMSApp(CMSApp):
    name = _("Catalog List")

    def get_urls(self, page=None, language=None, **kwargs):
        raise ImproperlyConfigured("`CatalogListApp must implement method `get_urls`.")


class CatalogSearchCMSApp(CMSApp):
    name = _("Catalog Search")

    def get_urls(self, page=None, language=None, **kwargs):
        raise ImproperlyConfigured("`CatalogSearchCMSApp` must implement method `get_urls`.")


class OrderCMSApp(CMSApp):
    name = _("View Orders")
    cache_placeholders = False

    def get_urls(self, page=None, language=None, **kwargs):
        from django.conf.urls import url
        from shop.views.order import OrderView

        if page and page.reverse_id == 'shop-order-last':
            return [
                url(r'^$', OrderView.as_view(many=False, is_last=True)),
            ]
        return [
            url(r'^$', OrderView.as_view()),
            url(r'^(?P<slug>[\w-]+)/?$', OrderView.as_view(many=False)),
        ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db.models.query import QuerySet
from django.utils.cache import add_never_cache_headers
from rest_framework import viewsets
from rest_framework.decorators import list_route
from rest_framework.response import Response
from shop.models.cart import CartModel, CartItemModel
from shop.serializers.cart import CartSerializer, CartItemSerializer, WatchSerializer, WatchItemSerializer


class BaseViewSet(viewsets.ModelViewSet):
    def get_queryset(self):
        try:
            cart = CartModel.objects.get_from_request(self.request)
            if self.kwargs.get(self.lookup_field):
                # we're interest only into a certain cart item
                return CartItemModel.objects.filter(cart=cart)
            # otherwise the CartSerializer will show its detail view and list all its cart items
            return cart
        except CartModel.DoesNotExist:
            return CartModel()

    def paginate_queryset(self, queryset):
        if isinstance(queryset, QuerySet):
            return super(BaseViewSet, self).paginate_queryset(queryset)

    def get_serializer(self, *args, **kwargs):
        kwargs.update(context=self.get_serializer_context(), label=self.serializer_label)
        many = kwargs.pop('many', False)
        if many or self.item_serializer_class is None:
            return self.serializer_class(*args, **kwargs)
        return self.item_serializer_class(*args, **kwargs)

    def finalize_response(self, request, response, *args, **kwargs):
        """Set HTTP headers to not cache this view"""
        if self.action != 'render_product_summary':
            add_never_cache_headers(response)
        return super(BaseViewSet, self).finalize_response(request, response, *args, **kwargs)


class CartViewSet(BaseViewSet):
    serializer_label = 'cart'
    serializer_class = CartSerializer
    item_serializer_class = CartItemSerializer

    @list_route(methods=['get'])
    def update_caption(self, request):
        cart = self.get_queryset()
        if cart:
            cart.update(request)
            caption = cart.get_caption_data()
        else:
            caption = CartModel.get_default_caption_data()
        return Response(caption)


class WatchViewSet(BaseViewSet):
    serializer_label = 'watch'
    serializer_class = WatchSerializer
    item_serializer_class = WatchItemSerializer



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from collections import OrderedDict
from rest_framework import serializers


class OrderedDictField(serializers.Field):
    """
    Serializer field which transparently bypasses the internal representation of an OrderedDict.
    """
    def to_representation(self, obj):
        return OrderedDict(obj)

    def to_internal_value(self, data):
        return OrderedDict(data)


class JSONSerializerField(serializers.Field):
    """
    Serializer field which transparently bypasses its object instead of serializing/deserializing.
    """
    def to_representation(self, obj):
        return obj

    def to_internal_value(self, data):
        return data
